// AO40RcvDoc.cpp : implementation of the CAO40RcvDoc class
//
//////////////////////////////////////////////////////////////////////
// Copyright 2000.    Moe Wheatley AE4JY  <ae4jy@qsl.net>
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "AO40Rcv.h"

#include "AO40RcvDoc.h"
#include "AO40RcvView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define DATABLOCK_SIZE 514
static const WORD AMSAT_CYC2_TBL[256] = {
	0x0000, 0x1021, 0x2042, 0x3063, 0x4084, 0x50a5, 0x60c6, 0x70e7, 0x8108, 0x9129, 0xa14a, 0xb16b, 0xc18c, 0xd1ad, 0xe1ce, 0xf1ef, 
	0x1231, 0x0210, 0x3273, 0x2252, 0x52b5, 0x4294, 0x72f7, 0x62d6, 0x9339, 0x8318, 0xb37b, 0xa35a, 0xd3bd, 0xc39c, 0xf3ff, 0xe3de, 
	0x2462, 0x3443, 0x0420, 0x1401, 0x64e6, 0x74c7, 0x44a4, 0x5485, 0xa56a, 0xb54b, 0x8528, 0x9509, 0xe5ee, 0xf5cf, 0xc5ac, 0xd58d, 
	0x3653, 0x2672, 0x1611, 0x0630, 0x76d7, 0x66f6, 0x5695, 0x46b4, 0xb75b, 0xa77a, 0x9719, 0x8738, 0xf7df, 0xe7fe, 0xd79d, 0xc7bc, 
	0x48c4, 0x58e5, 0x6886, 0x78a7, 0x0840, 0x1861, 0x2802, 0x3823, 0xc9cc, 0xd9ed, 0xe98e, 0xf9af, 0x8948, 0x9969, 0xa90a, 0xb92b, 
	0x5af5, 0x4ad4, 0x7ab7, 0x6a96, 0x1a71, 0x0a50, 0x3a33, 0x2a12, 0xdbfd, 0xcbdc, 0xfbbf, 0xeb9e, 0x9b79, 0x8b58, 0xbb3b, 0xab1a, 
	0x6ca6, 0x7c87, 0x4ce4, 0x5cc5, 0x2c22, 0x3c03, 0x0c60, 0x1c41, 0xedae, 0xfd8f, 0xcdec, 0xddcd, 0xad2a, 0xbd0b, 0x8d68, 0x9d49, 
	0x7e97, 0x6eb6, 0x5ed5, 0x4ef4, 0x3e13, 0x2e32, 0x1e51, 0x0e70, 0xff9f, 0xefbe, 0xdfdd, 0xcffc, 0xbf1b, 0xaf3a, 0x9f59, 0x8f78, 
	0x9188, 0x81a9, 0xb1ca, 0xa1eb, 0xd10c, 0xc12d, 0xf14e, 0xe16f, 0x1080, 0x00a1, 0x30c2, 0x20e3, 0x5004, 0x4025, 0x7046, 0x6067, 
	0x83b9, 0x9398, 0xa3fb, 0xb3da, 0xc33d, 0xd31c, 0xe37f, 0xf35e, 0x02b1, 0x1290, 0x22f3, 0x32d2, 0x4235, 0x5214, 0x6277, 0x7256, 
	0xb5ea, 0xa5cb, 0x95a8, 0x8589, 0xf56e, 0xe54f, 0xd52c, 0xc50d, 0x34e2, 0x24c3, 0x14a0, 0x0481, 0x7466, 0x6447, 0x5424, 0x4405, 
	0xa7db, 0xb7fa, 0x8799, 0x97b8, 0xe75f, 0xf77e, 0xc71d, 0xd73c, 0x26d3, 0x36f2, 0x0691, 0x16b0, 0x6657, 0x7676, 0x4615, 0x5634, 
	0xd94c, 0xc96d, 0xf90e, 0xe92f, 0x99c8, 0x89e9, 0xb98a, 0xa9ab, 0x5844, 0x4865, 0x7806, 0x6827, 0x18c0, 0x08e1, 0x3882, 0x28a3, 
	0xcb7d, 0xdb5c, 0xeb3f, 0xfb1e, 0x8bf9, 0x9bd8, 0xabbb, 0xbb9a, 0x4a75, 0x5a54, 0x6a37, 0x7a16, 0x0af1, 0x1ad0, 0x2ab3, 0x3a92, 
	0xfd2e, 0xed0f, 0xdd6c, 0xcd4d, 0xbdaa, 0xad8b, 0x9de8, 0x8dc9, 0x7c26, 0x6c07, 0x5c64, 0x4c45, 0x3ca2, 0x2c83, 0x1ce0, 0x0cc1, 
	0xef1f, 0xff3e, 0xcf5d, 0xdf7c, 0xaf9b, 0xbfba, 0x8fd9, 0x9ff8, 0x6e17, 0x7e36, 0x4e55, 0x5e74, 0x2e93, 0x3eb2, 0x0ed1, 0x1ef0
};


/////////////////////////////////////////////////////////////////////////////
// CAO40RcvDoc

IMPLEMENT_DYNCREATE(CAO40RcvDoc, CDocument)

BEGIN_MESSAGE_MAP(CAO40RcvDoc, CDocument)
	//{{AFX_MSG_MAP(CAO40RcvDoc)
		// NOTE - the ClassWizard will add and remove mapping macros here.
		//    DO NOT EDIT what you see in these blocks of generated code!
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CAO40RcvDoc construction/destruction

CAO40RcvDoc::CAO40RcvDoc()
{
	// TODO: add one-time construction code here

}

CAO40RcvDoc::~CAO40RcvDoc()
{
}

BOOL CAO40RcvDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;
	memset(&m_lfRcvFont, 0, sizeof m_lfRcvFont);
	if(!ReadFromRegistry())		// if registry read fails, load defaults
	{
		m_PlotType = TAB_SIG_VIEW0;
		m_InGroup = 0;
		m_BlkOffset = 0;
		m_SockDataFormat = FORMAT_P3T;
		m_QualifyCRC = TRUE;
		m_UseABlocks = TRUE;
		m_UseEBlocks = TRUE;
		m_RawLogOn = FALSE;
		m_TlmLogOn = FALSE;
		m_NetOn = FALSE;
		m_UseWaveOut = FALSE;
		m_UseTimeStamp = FALSE;
		m_USBMode = TRUE;
		m_AutoViewToggle = FALSE;
		m_InRawName = _T("No File Selected");
		m_InWaveName = _T("No File Selected");
		m_OutWavePath = _T(" ");
		m_InRawPath = _T(" ");
		m_InWavePath = _T(" ");
		m_CallSign = _T("CallNotSet");
		m_TelemPath = _T("C:\\");
		m_UPString = _T("UP");
		m_DNString = _T("DN");
		memset(&m_lfRcvFont, 0, sizeof m_lfRcvFont);
		m_lfRcvFont.lfHeight = -8;
		lstrcpy(m_lfRcvFont.lfFaceName, _T("Courier New"));
		m_lfRcvFont.lfOutPrecision = OUT_TT_PRECIS;
		m_lfRcvFont.lfClipPrecision = CLIP_DEFAULT_PRECIS;
		m_lfRcvFont.lfQuality = PROOF_QUALITY;
		m_lfRcvFont.lfPitchAndFamily = FF_SWISS | VARIABLE_PITCH;
		m_crRcvBackColor = RGB(0,0,0);
		m_crRcvTextColor = RGB(0,255,0);
		m_NetIP = 0x7F000001;
		m_NetSocket = 1024;
		m_ComPort = 0;
		m_RigModeGroup = 0;
		m_StepSize = 10;
		m_PortParams = (3<<16) + 0 + 1;
		m_CenterFrequency = 1600;
		m_SoundCard = -1;
		m_WaveStartOffset = 0;
		m_BitAdjust = 0;
		m_WaveAppend = FALSE;
		m_SNRFilterFactor = 215;
	}
//
	if( m_RigModeGroup<0 || m_RigModeGroup>255)
		m_RigModeGroup = 0;
	if( m_CenterFrequency<800 || m_CenterFrequency>2500)
		m_CenterFrequency = 1600;
	if( m_StepSize<10 || m_StepSize>100)
		m_StepSize = 10;
	if( m_BitAdjust<-150 || m_BitAdjust>150)
		m_BitAdjust = 0;
	
	ClearScreens();
	FindMaxBlocks();
	if (!AfxSocketInit())
	{
		m_NetworkAvailable = FALSE;
		if(m_NetOn)
		{
			AfxMessageBox(IDP_SOCKETS_INIT_FAILED);
			m_NetOn = FALSE;
		}
	}
	else
	{
		m_NetworkAvailable = TRUE;
	}
	m_DopplerShift = 0.0;
	m_NetConnected = FALSE;
	m_NetStatus = NET_OFF;
	m_RecvState = STATE_STOPPED;

	// Set the socket dialog pointers
	m_ConnectSocket.SetParent(this);
	m_ListenSocket.SetParent(this);
	if(m_NetworkAvailable && m_NetOn)
		StartSocket();
	return TRUE;
}

void CAO40RcvDoc::OnCloseDocument() 
{
	m_ListenSocket.Close();
	WriteToRegistry();
	CDocument::OnCloseDocument();
}


/////////////////////////////////////////////////////////////////////////////
// CAO40RcvDoc serialization

void CAO40RcvDoc::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
		// TODO: add storing code here
	}
	else
	{
		// TODO: add loading code here
	}
}

BOOL CAO40RcvDoc::WriteToRegistry()
{
BOOL bSpare = FALSE;
INT iSpare = 0;
double dSpare = 0.0;
CString sSpare = _T(" ");
CWinApp* pApp = AfxGetApp();
CString str;
//save BOOL's
	str.Format("%i:%i:%i:%i:%i:%i:%i:%i:%i:%i:%i:%i:%i:%i:%i",
		m_RawLogOn,
		m_QualifyCRC,
		m_UseEBlocks,
		m_UseABlocks,
		m_NetOn,
		m_TlmLogOn,
		m_UseWaveOut,
		m_UseTimeStamp,
		m_AutoViewToggle,
		m_WaveAppend,
		bSpare,
		bSpare,
		bSpare,
		bSpare,
		bSpare
		);
	if(!pApp->WriteProfileString("Settings\\BOOLs", "BOOLs", str))
		return FALSE;
//save INT's
	str.Format("%i:%i:%i:%i:%i:%i:%i:%i:%i:%i:%i:%i:%i:%i:%i:%i:%i:%i:%i:%i",
		m_InGroup,
		m_BlkOffset,
		m_PlotType,
		m_ComPort,
		m_USBMode,
		m_CenterFrequency,
		m_PortParams,
		m_NetIP,
		m_NetSocket,
		m_RigModeGroup,
		m_SoundCard,
		m_SockDataFormat,
		m_StepSize,
		m_WaveStartOffset,
		m_BitAdjust,
		m_SNRFilterFactor,
		iSpare,
		iSpare,
		iSpare,
		iSpare
		);
	if(!pApp->WriteProfileString("Settings\\INTs", "INTs", str))
		return FALSE;
// save colors	
	str.Format("%i:%i",
		m_crRcvBackColor,
		m_crRcvTextColor);
	if(!pApp->WriteProfileString("Settings\\Colors", "Colors", str))
		return FALSE;
//save fonts
	str.Format("%li:%li:%i:%i:%i:%i:%i:%i:%i:%i",
		m_lfRcvFont.lfWeight,
		m_lfRcvFont.lfHeight,
		m_lfRcvFont.lfOutPrecision,
		m_lfRcvFont.lfClipPrecision,
		m_lfRcvFont.lfQuality,
		m_lfRcvFont.lfItalic,
		m_lfRcvFont.lfUnderline,
		m_lfRcvFont.lfStrikeOut,
		m_lfRcvFont.lfCharSet,
		m_lfRcvFont.lfPitchAndFamily);
	if(!pApp->WriteProfileString("Settings\\Fonts", "TextFont", m_lfRcvFont.lfFaceName))
		return FALSE;
	if(!pApp->WriteProfileString("Settings\\Fonts", "Rcv", str))
		return FALSE;

//save strings
	if(!pApp->WriteProfileString("Settings\\Strings", "RawName", m_InRawName))
		return FALSE;
	if(!pApp->WriteProfileString("Settings\\Strings", "RawPath", m_InRawPath))
		return FALSE;
	if(!pApp->WriteProfileString("Settings\\Strings", "WaveName", m_InWaveName))
		return FALSE;
	if(!pApp->WriteProfileString("Settings\\Strings", "WavePath", m_InWavePath))
		return FALSE;
	if(!pApp->WriteProfileString("Settings\\Strings", "Callsign", m_CallSign))
		return FALSE;
	if(!pApp->WriteProfileString("Settings\\Strings", "OutWavePath", m_OutWavePath))
		return FALSE;
	if(!pApp->WriteProfileString("Settings\\Strings", "TelemPath", m_TelemPath))
		return FALSE;
	if(!pApp->WriteProfileString("Settings\\Strings", "UPString", m_UPString))
		return FALSE;
	if(!pApp->WriteProfileString("Settings\\Strings", "DNString", m_DNString))
		return FALSE;
	return TRUE;
}

BOOL CAO40RcvDoc::ReadFromRegistry()
{
BOOL bSpare = FALSE;
INT iSpare = 0;
double dSpare = 0.0;
CString sSpare = _T(" ");
CWinApp* pApp = AfxGetApp();
CString str;
INT cRead;
INT x1,x2,x3,x4,x5,x6,x7,x8;
	str = pApp->GetProfileString ("Settings\\BOOLs", "BOOLs");
	if(str.IsEmpty ())
		return FALSE;
// Get BOOLs
	cRead = _stscanf (str, "%i:%i:%i:%i:%i:%i:%i:%i:%i:%i:%i:%i:%i:%i:%i",
		&m_RawLogOn,
		&m_QualifyCRC,
		&m_UseEBlocks,
		&m_UseABlocks,
		&m_NetOn,
		&m_TlmLogOn,
		&m_UseWaveOut,
		&m_UseTimeStamp,
		&m_AutoViewToggle,
		&m_WaveAppend,
		&bSpare,
		&bSpare,
		&bSpare,
		&bSpare,
		&bSpare
		);
	if(cRead != 15)
		return FALSE;
//Get INT's
	str = pApp->GetProfileString ("Settings\\INTs", "INTs");
	if(str.IsEmpty ())
		return FALSE;
	cRead = _stscanf(str, "%i:%i:%i:%i:%i:%i:%i:%i:%i:%i:%i:%i:%i:%i:%i:%i:%i:%i:%i:%i",
		&m_InGroup,
		&m_BlkOffset,
		&m_PlotType,
		&m_ComPort,
		&m_USBMode,
		&m_CenterFrequency,
		&m_PortParams,
		&m_NetIP,
		&m_NetSocket,
		&m_RigModeGroup,
		&m_SoundCard,
		&m_SockDataFormat,
		&m_StepSize,
		&m_WaveStartOffset,
		&m_BitAdjust,
		&m_SNRFilterFactor,
		&iSpare,
		&iSpare,
		&iSpare,
		&iSpare		
		);
	if(cRead != 20)
		return FALSE;
//Get colors
	str = pApp->GetProfileString ("Settings\\Colors", "Colors");
	if(str.IsEmpty ())
		return FALSE;
	cRead = _stscanf( str, "%i:%i",
		&m_crRcvBackColor,
		&m_crRcvTextColor);
	if(cRead != 2)
		return FALSE;

//get fonts
	str = pApp->GetProfileString("Settings\\Fonts", "TextFont");
	if(str.IsEmpty ())
		return FALSE;
	_tcscpy(m_lfRcvFont.lfFaceName,str);
	str = pApp->GetProfileString("Settings\\Fonts", "Rcv");
	if(str.IsEmpty ())
		return FALSE;
	cRead = _stscanf( str, "%i:%i:%d:%i:%i:%i:%i:%i:%i:%i",
			&m_lfRcvFont.lfWeight, 
			&m_lfRcvFont.lfHeight, 
			&x1,&x2,&x3,&x4,&x5,&x6,&x7,&x8);
	if(cRead != 10)
		return FALSE;
	m_lfRcvFont.lfOutPrecision = x1;
	m_lfRcvFont.lfClipPrecision = x2;
	m_lfRcvFont.lfQuality = x3;
	m_lfRcvFont.lfItalic = x4;
	m_lfRcvFont.lfUnderline = x5;
	m_lfRcvFont.lfStrikeOut = x6;
	m_lfRcvFont.lfCharSet = x7;
	m_lfRcvFont.lfPitchAndFamily = x8;

//get  strings
	m_InRawName = pApp->GetProfileString("Settings\\Strings", "RawName",_T("No File Selected"));
	if(m_InRawName.IsEmpty ())
		return FALSE;
	m_InRawPath = pApp->GetProfileString("Settings\\Strings", "RawPath",_T(" "));
	if(m_InRawPath.IsEmpty ())
		return FALSE;
	m_InWaveName = pApp->GetProfileString("Settings\\Strings", "WaveName",_T("No File Selected"));
	if(m_InWaveName.IsEmpty ())
		return FALSE;
	m_InWavePath = pApp->GetProfileString("Settings\\Strings", "WavePath",_T(" "));
	if(m_InWavePath.IsEmpty ())
		return FALSE;
	m_CallSign = pApp->GetProfileString("Settings\\Strings", "Callsign",_T("CallNotSet"));
	if(m_CallSign.IsEmpty ())
		return FALSE;
	m_OutWavePath = pApp->GetProfileString("Settings\\Strings", "OutWavePath",_T(" "));
	if(m_CallSign.IsEmpty ())
		return FALSE;
	m_TelemPath = pApp->GetProfileString("Settings\\Strings", "TelemPath",_T("C:\\"));
	if(m_TelemPath.IsEmpty ())
		return FALSE;
	m_UPString = pApp->GetProfileString("Settings\\Strings", "UPString",_T("UP"));
	if(m_UPString.IsEmpty ())
		return FALSE;
	m_DNString = pApp->GetProfileString("Settings\\Strings", "DNString",_T("DN"));
	if(m_DNString.IsEmpty ())
		return FALSE;
	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CAO40RcvDoc diagnostics

#ifdef _DEBUG
void CAO40RcvDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CAO40RcvDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CAO40RcvDoc commands
BOOL CAO40RcvDoc::WriteToRawFile()
{
CFile cFile;
BOOL err = FALSE;
CString sPath;
CTime time = CTime::GetCurrentTime();
//	sPath = m_TelemPath + time.FormatGmt(_T("T%y%m%d@"));
	sPath = m_TelemPath + time.Format(_T("T%y%m%d@"));
	sPath = sPath + m_CallSign + _T(".raw");
	if( !cFile.Open( sPath, CFile::modeRead) )
	{
		if(!cFile.Open( sPath, CFile::modeCreate))
			err = TRUE;
	}
	cFile.Close();
	if( cFile.Open( sPath, CFile::modeWrite) && !err )
	{
		cFile.SeekToEnd();
		cFile.Write( m_RawDataBlock,512);
	}
	return err;
}

BOOL CAO40RcvDoc::WriteToTlmFile()
{
CFile cFile;
BOOL err = FALSE;
CString sPath;
CTime time = CTime::GetCurrentTime();
//	sPath = m_TelemPath + time.FormatGmt(_T("T%y%m%d@"));
	sPath = m_TelemPath + time.Format(_T("T%y%m%d@"));
	sPath = sPath + m_CallSign + _T(".tlm");
	if( !cFile.Open( sPath, CFile::modeRead) )
	{
		if(!cFile.Open( sPath, CFile::modeCreate))
			err = TRUE;
	}
	cFile.Close();
	if( cFile.Open( sPath, CFile::modeWrite) && !err )
	{
		cFile.SeekToEnd();
		cFile.Write( m_RawDataBlock,512);
	}
	return err;
}

INT CAO40RcvDoc::ReadRawFile()
{
CFile cFile;
BOOL err = FALSE;
INT val;
CHAR pBuf[512];
	if( cFile.Open( m_InRawPath, CFile::modeRead) )
	{
		val = cFile.Seek( 512*m_BlkOffset, CFile::begin );
		if( val != (m_BlkOffset*512) )
		{
			err = TRUE;
		}
		else 
		{
			val = cFile.Read( pBuf , 512);
			if( val != 512 )
				err = TRUE;
		}
		cFile.Close();
	}
	else
	{
		err = TRUE;
	}
	if( !err )
	{
		DecodeBlock( pBuf, FALSE);
	}
	return err;
}

BOOL CAO40RcvDoc::DecodeBlock(char* pBuf, BOOL crcok)
{
INT i;
BOOL error = FALSE;
BOOL sanitycrcok;

	for(i=0; i<514;i++)					//make local copy of raw data
		m_RawDataBlock[i] = pBuf[i];
	if( m_InGroup != INPUT_RAW )
	{
		if(m_NetConnected)
			SendNetworkData();
		if( CalcCRC(m_RawDataBlock) == 0)
			sanitycrcok = TRUE;
		else
			sanitycrcok = FALSE;
		if( !crcok || !sanitycrcok)
			m_RawDataBlock[0] |= (1<<7);	//set bit 7 of byte 0
		if(m_RawLogOn)
			error = WriteToRawFile();
		if(m_TlmLogOn & crcok & sanitycrcok)
			error = WriteToTlmFile();
	}
	else	// here if reading raw data from file instead of from decoder
	{
		if( m_RawDataBlock[0] & (1<<7) )
			crcok = FALSE;
		else
			crcok = TRUE;
	}
	if( !m_QualifyCRC || crcok)
	{
		m_TLMdecode.CopyData(m_RawDataBlock);	//pass data block to decoder
		m_TLMdecode.MakeRawStr( m_RawStr, crcok);
		switch( m_RawDataBlock[0] & 0x7F )
		{
			case 'A':
				if(m_UseABlocks)
				{
					m_TLMdecode.MakeStatusAStr(m_StatusStr, crcok);
					m_TLMdecode.MakeTempStr(m_TempStr, crcok);
					m_TLMdecode.MakeNavStr(m_NavStr, crcok);
					m_TLMdecode.MakePwrStr(m_PwrStr, crcok);
					m_TLMdecode.MakeMatrixStr(m_MatrixStr, crcok);
				}
				break;
			case 'E':
				if(m_UseEBlocks)
				{
					m_TLMdecode.MakeStatusEStr(m_StatusStr, crcok);
					m_TLMdecode.MakeTempStr(m_TempStr, crcok);
					m_TLMdecode.MakeNavStr(m_NavStr, crcok);
					m_TLMdecode.MakePwrStr( m_PwrStr, crcok);
					m_TLMdecode.MakeMatrixStr(m_MatrixStr, crcok);
				}
				break;
			case 'K':
				m_TLMdecode.MakeMailStr(m_MailKStr, crcok);
				break;
			case 'L':
				m_TLMdecode.MakeMailStr( m_MailLStr, crcok);
				break;
			case 'M':
				m_TLMdecode.MakeMailStr(m_MailMStr, crcok);
				break;
			case 'N':
				m_TLMdecode.MakeMailStr(m_MailNStr, crcok);
				break;
		}
	}
	return error;
}

CString CAO40RcvDoc::GetOutputText()
{
	switch(m_PlotType)
	{
		case TAB_STAT_VIEW1:
			return m_StatusStr;
			break;
		case TAB_MATRIX_VIEW2:
			return m_MatrixStr;
			break;
		case TAB_NAV_VIEW3:
			return m_NavStr;
			break;
		case TAB_PWR_VIEW4:
			return m_PwrStr;
			break;
		case TAB_TEMP_VIEW5:
			return m_TempStr;
			break;
		case TAB_MAIL_VIEW6:
			m_MailStr = "";
			m_MailStr = m_MailStr+m_MailKStr+m_MailLStr+m_MailMStr+m_MailNStr;
			return m_MailStr;
			break;
		case TAB_RAW_VIEW7:
			return m_RawStr;
			break;
		default:
			return "";
			break;
	}

}

void CAO40RcvDoc::ClearScreens()
{
	m_NavStr = "";
	m_MatrixStr = "";
	m_PwrStr = "";
	m_TempStr = "";
	m_MailStr = "";
	m_MailKStr = "";
	m_MailLStr = "";
	m_MailMStr = "";
	m_MailNStr = "";
	m_RawStr = "";
	m_StatusStr = "";
}

void CAO40RcvDoc::FindMaxBlocks(void)
{
CFile cFile;
	if( cFile.Open( m_InRawPath, CFile::modeRead) )
	{
		m_NumBlocksInFile = cFile.GetLength()/512;
		if(m_NumBlocksInFile < 1)
		{
			m_InRawPath = "";
			m_NumBlocksInFile = 0;
		}
	}
	else
		m_NumBlocksInFile = 0;
	if( m_BlkOffset >= m_NumBlocksInFile )
		m_BlkOffset = m_NumBlocksInFile-1;

}
/////////////////////////////////////////////////////////////
// Returns the 16 bit CRC of the specified data buffer.
////////////////////////////////////////////////////////////
WORD CAO40RcvDoc::CalcCRC(BYTE *buf)
{
WORD crc = 0xFFFF;
	for (INT i = 0; i < DATABLOCK_SIZE; i++)
		crc = (crc << 8) ^ AMSAT_CYC2_TBL[((crc >> 8) ^ buf[i]) & 0x00FF];
	return crc;
}

/////////////////////////////////////////////////////////////
//          Network Socket notification functions
/////////////////////////////////////////////////////////////
void CAO40RcvDoc::StartSocket()
{
	m_ListenSocket.Create(m_NetSocket);
	if(GetLastError() != 0)
	{
		m_NetworkAvailable = FALSE;
		m_NetStatus = NET_ERROR;
		return;
	}
	m_ListenSocket.Listen();
	if(GetLastError() != 0)
	{
		m_NetworkAvailable = FALSE;
		m_NetStatus = NET_ERROR;
		return;
	}
	m_NetStatus = NET_LISTENING;
}


void CAO40RcvDoc::SendNetworkFreq(INT freq)
{
INT iSent;
	if(	m_NetConnected && (m_SockDataFormat == FORMAT_FREQ) )
	{
		m_NetDataBlock[0] = 'F';	//Freq data
		m_NetDataBlock[1] = '=';
		m_NetDataBlock[2] = (BYTE)((freq&0xFF00) >> 8);
		m_NetDataBlock[3] = (BYTE)(freq&0x00FF);
		iSent = m_ConnectSocket.Send( m_NetDataBlock, 4);
		// Were we able to send it?
		if (iSent == SOCKET_ERROR)
			m_NetStatus = NET_ERROR;
		else
			m_NetStatus = NET_SEND;
		Sleep(0);
	}
}

void CAO40RcvDoc::SendNetworkData()
{
INT iSent;
INT i;
	if(m_SockDataFormat == FORMAT_FREQ )
		return;
	if(m_SockDataFormat == FORMAT_AO40 )
	{
		m_NetDataBlock[0] = 0x39;	//AO40 sync
		m_NetDataBlock[1] = 0x15;
		m_NetDataBlock[2] = 0xED;
		m_NetDataBlock[3] = 0x30;
		for(i=4; i<518; i++)
		{
			m_NetDataBlock[i] = m_RawDataBlock[ i-4 ];
		}
		iSent = m_ConnectSocket.Send( m_NetDataBlock, 518);
	}
	else
	{
		m_NetDataBlock[0] = 0x2F;	//P3T Sync
		m_NetDataBlock[1] = 0x8F;
		m_NetDataBlock[2] = 0x6E;
		m_NetDataBlock[3] = 0x4D;
		m_NetDataBlock[4] = 0x28;
		m_NetDataBlock[5] = 0x86;
		m_NetDataBlock[6] = 0x75;
		m_NetDataBlock[7] = 0x60;
		INT sent = 0;
		while(sent < 8)
		{
			iSent = m_ConnectSocket.Send( m_NetDataBlock+sent, 8-sent);
			sent += iSent;
		}
		sent = 0;
		Sleep(100);
		while(sent < 514)
		{
			iSent = m_ConnectSocket.Send( m_RawDataBlock+sent, 514-sent);
			sent += iSent;
		}
	}
	// Were we able to send it?
	if (iSent == SOCKET_ERROR)
		m_NetStatus = NET_ERROR;
	else
		m_NetStatus = NET_SEND;
}

void CAO40RcvDoc::OnSend()
{
	m_NetStatus = NET_CONNECTED;
}

void CAO40RcvDoc::OnReceive()
{
BYTE pBuf[1024];
	m_ConnectSocket.Receive(pBuf,1024,0);
	AfxMessageBox("Got Recv");
}

void CAO40RcvDoc::OnSocketClose()
{
	// Close the connected socket
	m_ConnectSocket.Close();
	m_NetConnected = FALSE;
	m_NetStatus = NET_LISTENING;
}

void CAO40RcvDoc::OnConnect()
{
}

void CAO40RcvDoc::OnAccept()
{
UINT port;
CString sPeer;
	if (m_NetConnected)
	{
		// Create a rejection socket
		CAsyncSocket sRjctSock;
		// Create a message to send
		CString strMsg = "Too many connections";
		// Accept using the rejection socket
		m_ListenSocket.Accept(sRjctSock);
		// Send the rejection message
		sRjctSock.Send(LPCTSTR(strMsg), strMsg.GetLength());
		// Close the socket
		sRjctSock.Close();
		AfxMessageBox("Too many connections, Rejected Connection");
	}
	else
	{
		// Accept the connection request
		m_ListenSocket.Accept(m_ConnectSocket);
		m_NetConnected = TRUE;
		m_NetStatus = NET_CONNECTED;
		if( m_ConnectSocket.GetPeerName( sPeer, port ) )
			m_ConnectedPeer = sPeer;
		else
			m_ConnectedPeer = "Unknown Peer";
	}
}

