// AO40RcvDoc.h : interface of the CAO40RcvDoc class
//
//////////////////////////////////////////////////////////////////////
// Copyright 2000.    Moe Wheatley AE4JY  <ae4jy@qsl.net>
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////

#if !defined(AFX_AO40RCVDOC_H__3B601988_EBD6_4D04_954A_3BB4F961B078__INCLUDED_)
#define AFX_AO40RCVDOC_H__3B601988_EBD6_4D04_954A_3BB4F961B078__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#define TAB_SIG_VIEW0 0			//TAB controls for data view display selection
#define TAB_STAT_VIEW1 1
#define TAB_MATRIX_VIEW2 2
#define TAB_NAV_VIEW3 3
#define TAB_PWR_VIEW4 4
#define TAB_TEMP_VIEW5 5
#define TAB_MAIL_VIEW6 6
#define TAB_RAW_VIEW7 7

#define DEST_SCREEN 0
#define DEST_FILE 1

#define INPUT_SOUND 0
#define INPUT_WAVE 1
#define INPUT_RAW 2

#define NET_OFF 0
#define NET_LISTENING 1
#define NET_CONNECTED 2
#define NET_SEND 3
#define NET_ERROR 4

#define STATE_STOPPED 0
#define STATE_NOSIG 1
#define STATE_SYNC 2
#define STATE_READ 3
#define STATE_CRCOK 4
#define STATE_CRCBAD 5
#define STATE_ERROR 6
#define STATE_TOOSLOW 7
#define STATE_FECOK 8
#define STATE_FECBAD 9

#define FORMAT_P3T 0
#define FORMAT_AO40 1
#define FORMAT_FREQ 2

#include "TLMdecode.h"
#include "DSocket.h"


class CAO40RcvDoc : public CDocument
{
protected: // create from serialization only
	CAO40RcvDoc();
	DECLARE_DYNCREATE(CAO40RcvDoc)

// Attributes
public:

// Operations
public:

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CAO40RcvDoc)
	public:
	virtual BOOL OnNewDocument();
	virtual void Serialize(CArchive& ar);
	virtual void OnCloseDocument();
	//}}AFX_VIRTUAL

// Implementation
public:
	double m_DopplerShift;
	CString m_ConnectedPeer;
	void StartSocket(void);
	BOOL m_NetworkAvailable;
	INT ReadRawFile(void);
	void FindMaxBlocks(void);
	void SendNetworkFreq(int freq);

	virtual ~CAO40RcvDoc();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif
	void ClearScreens();
	BOOL DecodeBlock(char* pBuf, BOOL crcok);
	CString GetOutputText();
	void OnSend();
	void OnReceive();
	void OnSocketClose();
	void OnConnect();
	void OnAccept();
//
	INT m_RecvState;
	INT m_NetStatus;
	INT m_NumBlocksInFile;

// Variables saved in registry
	BOOL m_RawLogOn;
	BOOL m_TlmLogOn;
	BOOL m_NetOn;
	BOOL m_QualifyCRC;
	BOOL m_UseEBlocks;
	BOOL m_UseABlocks;
	BOOL m_UseWaveOut;
	BOOL m_UseTimeStamp;
	BOOL m_USBMode;
	BOOL m_AutoViewToggle;
	BOOL m_WaveAppend;
	INT	m_InGroup;
	INT m_BlkOffset;
	INT m_PlotType;
	INT m_NetIP;
	INT m_NetSocket;
	INT m_ComPort;
	INT m_CenterFrequency;
	INT m_PortParams;
	INT m_RigModeGroup;
	INT m_SoundCard;
	INT m_SockDataFormat;
	INT m_StepSize;
	INT m_WaveStartOffset;
	INT m_BitAdjust;
	INT m_SNRFilterFactor;
	CString	m_InRawName;
	CString	m_InWaveName;
	CString	m_InRawPath;
	CString	m_InWavePath;
	CString	m_OutWavePath;
	CString	m_CallSign;
	CString	m_TelemPath;
	CString	m_UPString;
	CString	m_DNString;
	LOGFONT m_lfRcvFont;
	COLORREF m_crRcvBackColor;
	COLORREF m_crRcvTextColor;
//

protected:
	BOOL WriteToRegistry();
	BOOL ReadFromRegistry();

// Generated message map functions
protected:
	//{{AFX_MSG(CAO40RcvDoc)
		// NOTE - the ClassWizard will add and remove member functions here.
		//    DO NOT EDIT what you see in these blocks of generated code !
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
private:
	void SendNetworkData();
	WORD CalcCRC(BYTE *buf);
	BOOL WriteToRawFile();
	BOOL WriteToTlmFile();
	BYTE m_RawDataBlock[514];
	BYTE m_NetDataBlock[514+8];
	CString m_StatusStr;
	CString m_NavStr;
	CString m_MatrixStr;
	CString m_PwrStr;
	CString m_TempStr;
	CString m_MailStr;
	CString m_MailKStr;
	CString m_MailLStr;
	CString m_MailMStr;
	CString m_MailNStr;
	CString m_RawStr;
	CTLMdecode m_TLMdecode;
	BOOL m_NetConnected;
	CDSocket m_ListenSocket;
	CDSocket m_ConnectSocket;
};

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_AO40RCVDOC_H__3B601988_EBD6_4D04_954A_3BB4F961B078__INCLUDED_)
