// AO40RcvView.h : interface of the CAO40RcvView class
//
//////////////////////////////////////////////////////////////////////
// Copyright 2000.    Moe Wheatley AE4JY  <ae4jy@qsl.net>
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////

#if !defined(AFX_AO40RCVVIEW_H__183925AC_FA45_40D2_9D61_55ACBD79A5AB__INCLUDED_)
#define AFX_AO40RCVVIEW_H__183925AC_FA45_40D2_9D61_55ACBD79A5AB__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "ResizableFormView.h"
#include "PlotData.h"
#include "RcvEditCtrl.h"
#include "BProgressCtrl.h"

class CAO40RcvView : public CResizableFormView
{
protected: // create from serialization only
	CAO40RcvView();
	DECLARE_DYNCREATE(CAO40RcvView)

public:
	//{{AFX_DATA(CAO40RcvView)
	enum { IDD = IDD_AO40RCV_FORM };
	CStatic	m_CurFreqTxtCtrl;
	CStatic	m_FECErrorTxtCtrl;
	CStatic	m_FECStatTxtCtrl;
	CStatic	m_CRCStatTxtCtrl;
	CStatic	m_BitAdjTxtCtrl;
	CStatic	m_SNRCtrl;
	CStatic	m_DopplerCorrCtrl;
	CButton	m_DopplerUpCtrl;
	CButton	m_DopplerDnCtrl;
	CStatic	m_DopFreqCtrl;
	CButton	m_WavePauseCtrl;
	CButton	m_RightCtrl;
	CButton	m_LeftCtrl;
	CButton	m_StartStopCtrl;
	CStatic	m_NetStatusCtrl;
	CStatic	m_FileStatusCtrl;
	CStatic	m_BlkCountCtrl;
	CStatic	m_FreqTxtCntrl;
	CTabCtrl	m_TabCtrl;
	BOOL	m_AFCOn;
	//}}AFX_DATA
	CBProgressCtrl m_CRCProgCtrl;
	CBProgressCtrl m_FECProgCtrl;
	CRcvEditCtrl m_RxTextCtrl;
	CStatic	m_DebugCtrl3;
	CStatic	m_DebugCtrl2;
	CStatic	m_DebugCtrl1;

// Attributes
public:
	CAO40RcvDoc* GetDocument();

// Operations
public:

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CAO40RcvView)
	public:
	virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	virtual void OnInitialUpdate(); // called first time after construct
	virtual void OnDraw(CDC* pDC);
	//}}AFX_VIRTUAL

// Implementation
public:
	void SetTABExternal(INT tab);
	void ExtStartStop();
	void ExtLeftKey();
	void ExtRightKey();
	void SetDopplerCorrection(INT value);
	void SetDopplerMode(CAO40RcvDoc* pDoc);
	void InitTlmRead(void);
	void SetButtonTypes(void);
	void UpdateNetStatus(CAO40RcvDoc* pDoc);
	void StopProcess();
	virtual ~CAO40RcvView();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:
// Generated message map functions
protected:
	//{{AFX_MSG(CAO40RcvView)
	afx_msg void OnSelchangeTab(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnDestroy();
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnCleardata();
	afx_msg void OnAfcon();
	afx_msg void OnEditCopy();
	afx_msg void OnStartstop();
	afx_msg void OnLeftbutton();
	afx_msg void OnRightbutton();
	afx_msg void OnWavepause();
	afx_msg void OnDopdn();
	afx_msg void OnDopup();
	afx_msg void OnDeltaposBitadjspin(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnZerosnrbutton();
	afx_msg void OnMouseMove(UINT nFlags, CPoint point);
	//}}AFX_MSG
	afx_msg LRESULT OnDataRdy(UINT CorrectedBytes, LONG blocktype);
	afx_msg LRESULT OnFFTDataRdy(UINT x, LONG y);
	afx_msg LRESULT OnStatusChange(UINT x, LONG y);
	DECLARE_MESSAGE_MAP()
private:
	void Start();
	void Stop();
	void ChangeDopplerCorrection(INT value);
	void CheckWavePause(void);
	void NewFrameSize(CRect cr);

	BOOL m_DispFreeze;
	INT m_AutoViewTimer;
	BOOL m_WaveLogPaused;
	INT m_LastNetStatus;
	INT m_DopplerCorrection;
	INT m_BlksOK;
	INT m_FECBlksOK;
	INT m_BlksReceived;
	INT m_FECBlksReceived;
	INT m_ProgCounter;
	INT m_RxFreq;
	INT m_LastTab;
	INT m_Status;
	INT m_FECProgCounter;
	char m_BlockData[514];
	CPlotData* m_pCPlotData;
	CString m_Str;
	CRect m_FrameRect;
	CRect m_LastRect;

};

#ifndef _DEBUG  // debug version in AO40RcvView.cpp
inline CAO40RcvDoc* CAO40RcvView::GetDocument()
   { return (CAO40RcvDoc*)m_pDocument; }
#endif

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_AO40RCVVIEW_H__183925AC_FA45_40D2_9D61_55ACBD79A5AB__INCLUDED_)
