//////////////////////////////////////////////////////////////////////
// PskDet.cpp: implementation of the CBPSKDet class.
//////////////////////////////////////////////////////////////////////
// Copyright 2000.    Moe Wheatley AE4JY  <ae4jy@qsl.net>
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////
//

#include "stdafx.h"
#include "BpskDet.h"
#include "Bpsk400.h"
#include "Perform.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

// define some constants
#define SAMPLE_RATE 8000.0
#define SYMB_RATE 400.0
#define TSYM (1.0/SYMB_RATE+0.0)			// TSYM == symbol period

#define BYTE_SYNC 0x50505050
#define BLOCK_SYNC 0x3915ED30
#define BLOCK_SIZE 514
#define SYNC_LIMIT 250

#define MAXBLOCKSIZE 2048

#define FREQSEARCHTIME1 1*4		//Oneshot time limits for searching for Freq
#define FREQSEARCHTIME2 10*4
#define FREQSEARCHTIME3 30*4
#define FREQLIMIT 25.0

#define FECACTIVETIME 30*4		//time limit before saying FEC blocks aren't being rcv'd
#define FECDISPLAYTIME (30-5)*4	//time to display received FEC block

extern double DebugFP1;
extern double DebugFP2;
extern INT DebugINT;

static const WORD AMSAT_CYC2_TBL[256] = {
	0x0000, 0x1021, 0x2042, 0x3063, 0x4084, 0x50a5, 0x60c6, 0x70e7, 0x8108, 0x9129, 0xa14a, 0xb16b, 0xc18c, 0xd1ad, 0xe1ce, 0xf1ef, 
	0x1231, 0x0210, 0x3273, 0x2252, 0x52b5, 0x4294, 0x72f7, 0x62d6, 0x9339, 0x8318, 0xb37b, 0xa35a, 0xd3bd, 0xc39c, 0xf3ff, 0xe3de, 
	0x2462, 0x3443, 0x0420, 0x1401, 0x64e6, 0x74c7, 0x44a4, 0x5485, 0xa56a, 0xb54b, 0x8528, 0x9509, 0xe5ee, 0xf5cf, 0xc5ac, 0xd58d, 
	0x3653, 0x2672, 0x1611, 0x0630, 0x76d7, 0x66f6, 0x5695, 0x46b4, 0xb75b, 0xa77a, 0x9719, 0x8738, 0xf7df, 0xe7fe, 0xd79d, 0xc7bc, 
	0x48c4, 0x58e5, 0x6886, 0x78a7, 0x0840, 0x1861, 0x2802, 0x3823, 0xc9cc, 0xd9ed, 0xe98e, 0xf9af, 0x8948, 0x9969, 0xa90a, 0xb92b, 
	0x5af5, 0x4ad4, 0x7ab7, 0x6a96, 0x1a71, 0x0a50, 0x3a33, 0x2a12, 0xdbfd, 0xcbdc, 0xfbbf, 0xeb9e, 0x9b79, 0x8b58, 0xbb3b, 0xab1a, 
	0x6ca6, 0x7c87, 0x4ce4, 0x5cc5, 0x2c22, 0x3c03, 0x0c60, 0x1c41, 0xedae, 0xfd8f, 0xcdec, 0xddcd, 0xad2a, 0xbd0b, 0x8d68, 0x9d49, 
	0x7e97, 0x6eb6, 0x5ed5, 0x4ef4, 0x3e13, 0x2e32, 0x1e51, 0x0e70, 0xff9f, 0xefbe, 0xdfdd, 0xcffc, 0xbf1b, 0xaf3a, 0x9f59, 0x8f78, 
	0x9188, 0x81a9, 0xb1ca, 0xa1eb, 0xd10c, 0xc12d, 0xf14e, 0xe16f, 0x1080, 0x00a1, 0x30c2, 0x20e3, 0x5004, 0x4025, 0x7046, 0x6067, 
	0x83b9, 0x9398, 0xa3fb, 0xb3da, 0xc33d, 0xd31c, 0xe37f, 0xf35e, 0x02b1, 0x1290, 0x22f3, 0x32d2, 0x4235, 0x5214, 0x6277, 0x7256, 
	0xb5ea, 0xa5cb, 0x95a8, 0x8589, 0xf56e, 0xe54f, 0xd52c, 0xc50d, 0x34e2, 0x24c3, 0x14a0, 0x0481, 0x7466, 0x6447, 0x5424, 0x4405, 
	0xa7db, 0xb7fa, 0x8799, 0x97b8, 0xe75f, 0xf77e, 0xc71d, 0xd73c, 0x26d3, 0x36f2, 0x0691, 0x16b0, 0x6657, 0x7676, 0x4615, 0x5634, 
	0xd94c, 0xc96d, 0xf90e, 0xe92f, 0x99c8, 0x89e9, 0xb98a, 0xa9ab, 0x5844, 0x4865, 0x7806, 0x6827, 0x18c0, 0x08e1, 0x3882, 0x28a3, 
	0xcb7d, 0xdb5c, 0xeb3f, 0xfb1e, 0x8bf9, 0x9bd8, 0xabbb, 0xbb9a, 0x4a75, 0x5a54, 0x6a37, 0x7a16, 0x0af1, 0x1ad0, 0x2ab3, 0x3a92, 
	0xfd2e, 0xed0f, 0xdd6c, 0xcd4d, 0xbdaa, 0xad8b, 0x9de8, 0x8dc9, 0x7c26, 0x6c07, 0x5c64, 0x4c45, 0x3ca2, 0x2c83, 0x1ce0, 0x0cc1, 
	0xef1f, 0xff3e, 0xcf5d, 0xdf7c, 0xaf9b, 0xbfba, 0x8fd9, 0x9ff8, 0x6e17, 0x7e36, 0x4e55, 0x5e74, 0x2e93, 0x3eb2, 0x0ed1, 0x1ef0
};

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
CBPSKDet::CBPSKDet()
{
	m_RxFrequency = 1600;
	m_SampleFreq = SAMPLE_RATE;
	m_IQPhzIndex = 0;
	m_PhaseError = 0.0;
	m_PkPos = 0;
	m_NCOphzinc = (PI2*(double)m_RxFrequency/(double)m_SampleFreq);
	m_BitPhaseInc = 1.0/(double)m_SampleFreq;		//bit sampling period
	//Generate integer sin/cos table for faster frequency finding routine
	for(int i=0; i<256; i++)
	{
		m_Sin[i] = sin((double)i*PI2/256.);
		m_Cos[i] = cos((double)i*PI2/256.);
	}
	m_FilterGainFactor = 215;
}

CBPSKDet::~CBPSKDet()
{
}

/////////////////////////////////////////////////////////////////
//       Initialize BpskDet buffers and pointers
/////////////////////////////////////////////////////////////////
void CBPSKDet::Init( INT Fs, INT BlockSize, HWND hWnd )
{
INT i;
	m_SampleFreq = Fs;
	m_hWnd = hWnd;
	m_BlockSize = BlockSize;	//size data input buffer
//
	m_NCOphzinc = (PI2*(double)m_RxFrequency/(double)m_SampleFreq);
	m_BitPhaseInc = 1.0/(double)m_SampleFreq;		//bit sampling period
	ResetDetector();
	m_BlockSize = BlockSize;	//size data input buffer
	m_BitPhasePos = 0.0;
	m_BitAcc = 0;
	m_BitCnt = 0;
	m_ByteCnt = 0;
	m_DecodeCRCState = DLLSTAT_RXCRC_NO_SIGNAL;
	m_DecodeFECState = DLLSTAT_RXFEC_NO_SIGNAL;
	m_BitPhzErrCnt = 0;
// Init a bunch of "static" variables used in various member fuctions
	m_VcoPhz = 0.0;
	m_PkPos = 0;
	m_NewPkPos = 0;
	m_BitPos = 0;
	m_LastIQ.x = 0.0;
	m_LastIQ.y = 0.0;
	m_PhzErrAve = 0.0;
	m_DopplerRate = 0.0;
	m_LastPkPos = 0;
	m_CRCok = FALSE;
	for( i = 0; i<LPFIR_LEN; i++)
	{
		m_pQue[i].x = 0.0;
		m_pQue[i].y = 0.0;
	}
	m_FirState = LPFIR_LEN-1;
}


//////////////////////////////////////////////////////////////////////
// Called to get the symbol vectors
//////////////////////////////////////////////////////////////////////
void CBPSKDet::GetVectorData(long *VectData)
{
	for(INT i=0; i<16; i++)
		VectData[i] = m_IQPhaseArray[i];
}

//////////////////////////////////////////////////////////////////////
// Called to change the Rx frequency manually
// freq is the new center frequency
//////////////////////////////////////////////////////////////////////
void CBPSKDet::SetRXFrequency(INT freq)
{
	if( freq != m_RxFrequency  )
	{
		m_RxFrequency = freq;
		m_NCOphzinc = PI2*(double)freq/m_SampleFreq;
		m_PhzErrAve = 0.0;
		m_PhaseError = 0.0;
		m_DopplerRate = 0.0;
		m_DecodeCRCState = DLLSTAT_RXCRC_NO_SIGNAL;
		m_DecodeFECState = DLLSTAT_RXFEC_NO_SIGNAL;
	}
}

//////////////////////////////////////////////////////////////////////
//sets adjustment for off freq soundcards/wavefiles
//////////////////////////////////////////////////////////////////////
void CBPSKDet::SetBitAdj(INT Adj)
{
	m_BitPhaseInc = 1.0/(double)(m_SampleFreq+Adj);
}

//////////////////////////////////////////////////////////////////////
//try to counter effects of rig doppler control freq changes
//////////////////////////////////////////////////////////////////////
void CBPSKDet::AdjDopplerJmp(INT step)
{
	m_RxFrequency += step;
	m_NCOphzinc = PI2*(double)m_RxFrequency/m_SampleFreq;
}

//////////////////////////////////////////////////////////////////////
// Called to get the data block
//////////////////////////////////////////////////////////////////////
BOOL CBPSKDet::GetDataBlock( char* pData)
{
	for(INT i=0; i<BLOCK_SIZE; i++)
		pData[i] = m_DataBuf[i];
	return m_CRCok;
}

//////////////////////////////////////////////////////////////////////
// Called to reset the detector
//////////////////////////////////////////////////////////////////////
void CBPSKDet::ResetDetector()
{
	for(INT i=0; i<LPFIR_LEN; i++)
	{
		m_pQue[i].x = 0.0;	// fill delay buffer with zero
		m_pQue[i].y = 0.0;
	}
	for(i=0; i<16; i++)
	{
		m_IQPhaseArray[i] = 1;
	}
	for(i=0; i<22; i++)
	{
		m_SyncAve[i] = 0.0;				// initialize the array
	}
	m_BitAcc = 0;
	m_BitCnt = 0;
	m_ByteCnt = 0;
	m_DecodeCRCState = DLLSTAT_RXCRC_NO_SIGNAL;
	m_DecodeFECState = DLLSTAT_RXFEC_NO_SIGNAL;
	m_FECdecoder.Init();
	m_EnergyOut = 1.0;
	m_EnergyIn = 1.0;
	m_SNR = 0;
	m_PhaseError = 0.0;
	m_FreqSearchTimer = FREQSEARCHTIME1;
	m_FECActiveTimer = 0;
}

//////////////////////////////////////////////////////////////
//   Finds best guess at signal's center frequency
// this routine does an exhaustive(time/CPU consuming) search
// for the signal's center frequency
//////////////////////////////////////////////////////////////
int CBPSKDet::FindCenterFrequency(double* pIn)
{
double vcophz = m_VcoPhz;
_complex BaseBand[MAXBLOCKSIZE];
INT i,j,k;
_complex acc;
int IntPhase;
double Trialphzinc = PI2*700.0/m_SampleFreq;
double EnergyOut = 0.0;
double BestFreq;
int LastPosition = m_BlockSize - LPFIR_LEN;
//StartPerformance();
IntPhase =0;
	//look from 700 to 3300 Hz in 40Hz steps
	for(k=0; k<40; k++, Trialphzinc += (PI2*40.0/m_SampleFreq) )
	{
		vcophz = m_VcoPhz;
		for( i=0; i<m_BlockSize; i++ )
		{
			//Convert to baseband complex samples by mixing input sample with NCO's sin/cos
			IntPhase = (int)(vcophz*256.0/PI2);
			vcophz = vcophz + Trialphzinc;
			if(vcophz > PI2)		//handle 2 Pi wrap around
				vcophz -= PI2;
			BaseBand[i].x = pIn[i] * m_Cos[IntPhase];	//faster table lookup
			BaseBand[i].y = pIn[i] * m_Sin[IntPhase];
		}
		//Now for each frequency trial, trya all possible bit phases
		for( int TrialPhase=0; TrialPhase<20; TrialPhase++ )
		{
			double energy = 0.0;
			for( int mo=TrialPhase; mo<LastPosition; mo+=20 )
			{
				acc.x = 0.0; acc.y = 0.0;
				_complex* pData = &BaseBand[mo];
				for(j=0; j<LPFIR_LEN; j++)
				{
					acc.x += ( pData[j].x * LPfir_COEF[j] );
					acc.y += ( pData[j].y * LPfir_COEF[j] );
				}
				energy += (acc.x*acc.x + acc.y*acc.y);
			}
			if(energy>EnergyOut)
			{
				BestFreq = Trialphzinc;		//save maximum energy frequency
				EnergyOut = energy;
			}
		}
	}
//StopPerformance();
	return (INT)(0.5+((BestFreq)*m_SampleFreq/PI2 ) );
}

//////////////////////////////////////////////////////////////////////
// Main routine called to process the next block of data 'pIn'.
//////////////////////////////////////////////////////////////////////
INT CBPSKDet::ProcBPSKDet( double* pIn, BOOL UseFreqSearch)
{
double vcophz = m_VcoPhz;
INT i,j;
_complex acc;
const double* Kptr;
_complex* Firptr;
	if(UseFreqSearch)	//if auto freq is enabled
	{
		if(	m_FreqSearchTimer < 0)
		{	//if one shot timer timed out
			double f = FindCenterFrequency(pIn);
			if( fabs( f - (double)m_RxFrequency ) > FREQLIMIT )
			{	//if new search is a lot different from last frequency
				m_FreqSearchTimer = FREQSEARCHTIME1;
				if( (m_DecodeCRCState == DLLSTAT_RXCRC_NO_SIGNAL) &&
					(m_DecodeFECState == DLLSTAT_RXFEC_NO_SIGNAL) )
				{	//if not getting blocks then reset to new freq estimate
					m_RxFrequency = (int)f;
					m_NCOphzinc = PI2*f/m_SampleFreq;
					m_PhaseError = 0.0;
					m_DopplerRate = 0.0;
				}
			}
			else
			{	//if still in range just reset one shot
				m_FreqSearchTimer = FREQSEARCHTIME2;
			}
		}
		else
		{	//dec oneshot timer
			m_FreqSearchTimer--;
		}
	}
//DebugINT = m_FreqSearchTimer;
	if(m_FECActiveTimer>0)
	{	//create some FEC block decode state timing for status updates
		if(	m_FECActiveTimer == FECDISPLAYTIME)
			m_DecodeFECState = DLLSTAT_RXFEC_SYNC_SEARCH;
		if(--m_FECActiveTimer <= 0)
			m_DecodeFECState = DLLSTAT_RXFEC_NO_SIGNAL;
	}
//StartPerformance();
	for( i = 0; i<m_BlockSize; i++ )	// put new samples into Queue
	{
		double samp = pIn[i];
		m_EnergyIn = (1.0-1.0/16000.)*m_EnergyIn + (1.0/16000.)*(samp*samp);
//Generate complex sample by mixing input sample with NCO's sin/cos
//		vcophz = vcophz + m_NCOphzinc;
		vcophz = vcophz + m_NCOphzinc + m_PhaseError;
		if(vcophz > PI2)		//handle 2 Pi wrap around
			vcophz -= PI2;
		acc.x = samp * cos( vcophz );
		acc.y = samp * sin( vcophz );
//Matched?? Bit filter 
		m_pQue[m_FirState].x = acc.x;	//place in circular Queue
		m_pQue[m_FirState].y = acc.y;
		Firptr = m_pQue;
		Kptr = LPfir_COEF + LPFIR_LEN - m_FirState;
		acc.x = 0.0;
		acc.y = 0.0;
		for(j=0; j<	LPFIR_LEN;j++)
		{
			acc.x += ( (Firptr->x)*(*Kptr) );
			acc.y += ( (Firptr++->y)*(*Kptr++) );
		}
		if( --m_FirState < 0)
			m_FirState = LPFIR_LEN-1;
// Bit Timing synchronization
		if( SymbSync(acc) ) 
			DecodeSymb( acc );
	}
	m_VcoPhz = vcophz;
	m_RxFrequency = (INT)(0.5+((m_NCOphzinc + m_PhaseError)*m_SampleFreq/PI2 ) );
//	m_RxFrequency = (INT)(0.5+((m_NCOphzinc)*m_SampleFreq/PI2 ) );

	// calculate SNR
	double fgain = m_EnergyOut/(m_EnergyIn*m_FilterGainFactor);
//DebugFP2 = fgain;
	if( fgain<1.0 )
	{
		if( fgain>.1 )
		{
			double sn;
			sn = 20.0*(fgain - (2.0*400.0/8000.0) )/(1.0-fgain);
			m_SNR = (int)(100*log10(sn));
		}
		else
			m_SNR = -600;
	}
	else
		m_SNR = 600;

//StopPerformance();
	return (m_DecodeCRCState|m_DecodeFECState);
}

////////////////////////////////////////////////////////////
// Called with -1 to set current SNR to zero.
//Called with anything else to set it to a previously known value
////////////////////////////////////////////////////////////
double CBPSKDet::ZeroSNR(double FilterFactor)
{
	if(FilterFactor <= 0.0)
	{
		//if sn = 1
		//fgain = 3/21 so
		m_FilterGainFactor = (m_EnergyOut*21.0)/(m_EnergyIn*3.0);
	}
	else
	{
		m_FilterGainFactor = FilterFactor;
	}
	return m_FilterGainFactor;
}

//////////////////////////////////////////////////////////////////////
// calculates AFC value to adjust NCO frequency
//////////////////////////////////////////////////////////////////////
void CBPSKDet::CalcPhaseError( _complex IQ )
{
double samp;
#define ERR_K (1.0/14.1)	//make error into Hz.

#define ERR_K2 (SYMB_RATE*SAMPLE_RATE/PI2)	//make doppler rate into Hz/sec.
#define PLP_K2 50

#define PLP_K 175.0
#define P_GN (0.0004)
#define I_GN 7e-6
#define HYST 2.0

	if(IQ.y < 0.0)
		samp = -IQ.x;
	else
		samp = IQ.x;
// error is signal's "x" component
	m_PhzErrAve = (1.0-1.0/PLP_K)*m_PhzErrAve + ((1.0*ERR_K)/PLP_K)*samp;
	samp = m_NCOphzinc;
	if( (m_PhzErrAve > HYST) || (m_PhzErrAve < -HYST ) )
	{
		m_NCOphzinc = m_NCOphzinc + (m_PhzErrAve*I_GN);
		if(m_NCOphzinc < 0.157)
			m_NCOphzinc = 0.157;
		else
			if(m_NCOphzinc > 2.67)
				m_NCOphzinc = 2.67;
	}
	m_PhaseError = m_PhzErrAve*P_GN;
//DebugFP2 = m_PhzErrAve;
	m_DopplerRate = (1.0-1.0/PLP_K2)*m_DopplerRate + ((1.0*ERR_K2)/PLP_K2)*(m_NCOphzinc-samp);
}

//////////////////////////////////////////////////////////////////////
// Called at Fs rate to calculate the symbol sync position
// Returns TRUE if at center of symbol.
// Sums up the energy at each sample time, averages it, and picks the
//   sample time with the highest energy content.
//////////////////////////////////////////////////////////////////////
BOOL CBPSKDet::SymbSync(_complex sample)
{
BOOL Trigger;
double max;
double energy;
INT BitPos = m_BitPos;
	energy = (sample.x*sample.x) + (sample.y*sample.y);
	m_SyncAve[BitPos] = (1.0-1.0/200.0)*m_SyncAve[BitPos] + (1.0/200.0)*energy;
	if( BitPos == m_PkPos )	// see if at middle of symbol
		Trigger = TRUE;
	else
		Trigger = FALSE;
	if( BitPos == HALF_TBL[m_PkPos] )	//don't change pk pos until
		m_PkPos = m_NewPkPos;			// halfway into next bit.
	BitPos++;
	if(BitPos>=20)
		BitPos = 0;
	m_BitPhasePos += (m_BitPhaseInc);
	if( m_BitPhasePos >= TSYM )
	{									// here every symbol time
		m_BitPhasePos -= TSYM;			//keep phase bounded
		BitPos = 0;
		max = -1e10;
		for( INT i=0; i<20; i++)		//find maximum energy pk
		{
			energy = m_SyncAve[i];
			if( energy > max )
			{
				m_NewPkPos = i;
				max = energy;
			}
		}
	}
	m_BitPos = BitPos;
	return Trigger;
}

//////////////////////////////////////////////////////////////////////
//  Decode the new symbol sample
//////////////////////////////////////////////////////////////////////
void CBPSKDet::DecodeSymb(_complex newsamp)
{
_complex vect;
_complex nvect;
double energy;
ECstats stats;

	energy = newsamp.x*newsamp.x + newsamp.y*newsamp.y;
	m_EnergyOut = (1.0-1.0/800.)*m_EnergyOut + (1.0/800.)*(energy);
	//find DBPSK symbol vector
	vect.x = -(m_LastIQ.x*newsamp.x + m_LastIQ.y*newsamp.y);
	vect.y = (m_LastIQ.x*newsamp.y - newsamp.x*m_LastIQ.y);	//for use in the AFC
	m_LastIQ = newsamp;
	energy = sqrt(vect.x*vect.x + vect.y*vect.y);
	if( energy > 100.0 )	//display if over a threshold
	{
		double e = energy/1000.;
		nvect.x = (vect.y/e);
		nvect.y = (vect.x/e);
		CalcPhaseError( nvect );	//calc new phase error
	}
	else
	{
		nvect.x = 1.0;
		nvect.y = 1.0;
		m_DecodeCRCState = DLLSTAT_RXCRC_NO_SIGNAL;
		m_DecodeFECState = DLLSTAT_RXFEC_NO_SIGNAL;
		return;
	}
	m_IQPhaseArray[m_IQPhzIndex++] = (LONG)nvect.x;	//calc display vectors
	m_IQPhaseArray[m_IQPhzIndex++] = (LONG)nvect.y;
	m_IQPhzIndex &= 0x000F;		//mod 16 index

	int symbol = (int)(vect.x/256.);	//scale to keep from overflowing int

	//run regular CRC block decoding
	ProcessCRCsymbol( (symbol > 0) );
	//run FEC block decoding
	if( m_FECdecoder.ProcessFECSymbol( symbol, &stats ) )
	{	//here if got a FEC block good or bad
		m_FECActiveTimer = FECACTIVETIME;
		m_DecodeFECState = DLLSTAT_RXFEC_BLOCK_READ;
		m_FreqSearchTimer = FREQSEARCHTIME3;
		m_CRCok = stats.ok;
		m_FECdecoder.GetDataBlock(m_CRCok, m_DataBuf);
		short int crc;
		int errors = stats.RS0errors + stats.RS1errors;	//upper 16 bits
		errors = errors<<16;
		errors += stats.SYMerrors;	//lower 16 bits
		crc = CalcCRC(m_DataBuf,BLOCK_SIZE-2);
		if(!m_CRCok)
		{
			crc += 0x1234;	//make sure data CRC fails
			errors = 0;
		}
		m_DataBuf[512] = (crc>>8)&0x00FF;	//place CRC into block
		m_DataBuf[513] = crc&0x00FF;
		::PostMessage( m_hWnd, MSG_DATARDY, errors, FECBLOCK);
	}
}

//////////////////////////////////////////////////////////////////////
//  Decode CRC block data every 400Hz symbol sample
//////////////////////////////////////////////////////////////////////
INT CBPSKDet::ProcessCRCsymbol(BOOL bit)
{
	m_BitAcc = m_BitAcc<<1;//shift in bit
	if(bit)
		m_BitAcc |= 1;
	if( ++m_BitCnt >= 8 )
		m_BitCnt = 0;
	switch(	m_DecodeCRCState )
	{
		case DLLSTAT_RXCRC_NO_SIGNAL:		//look for IDLE sync
			if(m_BitAcc == BYTE_SYNC )
			{
				m_BitCnt = 0;
				m_ByteCnt = 0;
				m_DecodeCRCState = DLLSTAT_RXCRC_SYNC_SEARCH;
			}
			break;
		case DLLSTAT_RXCRC_SYNC_SEARCH:		//look for block sync
			if(m_BitAcc == BYTE_SYNC )	//swallow up more idle syncs
			{
				m_BitCnt = 0;
				m_ByteCnt = 0;
			}
			else
			{
				if( m_BitCnt == 0 )
				{
					if( ++m_ByteCnt >= SYNC_LIMIT) //if haven't gotten blk sync
					{								//timeout and try again
						m_ByteCnt = 0;
						m_DecodeCRCState = DLLSTAT_RXCRC_NO_SIGNAL;
					}
					if(m_BitAcc == BLOCK_SYNC)	//if got it
					{
						m_ByteCnt = 0;
						m_DecodeCRCState = DLLSTAT_RXCRC_BLOCK_READ;
						m_FreqSearchTimer = FREQSEARCHTIME3;
					}
				}
			}
			break;
		case DLLSTAT_RXCRC_BLOCK_READ:		//here filling up data block
			if( m_BitCnt == 0 )
			{
				if( m_ByteCnt < BLOCK_SIZE)
				{
					m_DataBuf[m_ByteCnt++] = m_BitAcc & 0xFF;
				}
				else	// here if got all bytes for this phz's block
				{
					if(	CalcCRC(m_DataBuf,BLOCK_SIZE) == 0)	//check CRC
						m_CRCok = TRUE;		//if good crc then kill otherphz
					else
						m_CRCok = FALSE;
					::PostMessage( m_hWnd, MSG_DATARDY, 0, CRCBLOCK);
					m_DecodeCRCState = DLLSTAT_RXCRC_SYNC_SEARCH;
					m_ByteCnt = 0;
				}
			}
			break;
	}
	return m_DecodeCRCState;
}

/////////////////////////////////////////////////////////////
// Returns the 16 bit CRC of the specified data buffer.
////////////////////////////////////////////////////////////
WORD CBPSKDet::CalcCRC(BYTE *buf, int length)
{
WORD crc = 0xFFFF;
	for (INT i = 0; i < length; i++)
		crc = (crc << 8) ^ AMSAT_CYC2_TBL[((crc >> 8) ^ buf[i]) & 0x00FF];
	return crc;
}




