// FECDecode.cpp: implementation of the CFECDecode class.
//////////////////////////////////////////////////////////////////////
// Moe Wheatley AE4JY  <ae4jy@qsl.net>
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////
//
//  This program is based on material largely developed by Phil Karn KA9Q
//  over the period 1997-2002.  Much of it can be found in libraries on 
//  http://www.ka9q.net/ao40/  
//				Copyright 1997 Phil Karn, KA9Q
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "FECDecode.h"
#include "math.h"
#include "Perform.h"

extern double DebugFP1;
extern double DebugFP2;
extern INT DebugINT;

//Reed Solomon Encoder constants
#define MM 8
#define NN 255
#define NROOTS 32

//Viterbi Encoder constants
#define	V27POLYA	0x6d
#define	V27POLYB	0x4f

#define ALPHA_TO (m_RS.alpha_to) 
#define INDEX_OF (m_RS.index_of)
#define GENPOLY (m_RS.genpoly)
#define FCR (m_RS.fcr)
#define PRIM (m_RS.prim)
#define IPRIM (m_RS.iprim)
#define PAD (m_RS.pad)
#define A0 (NN)

//Viterbi Butterfly macro definition
#define BFLY(i) {\
unsigned char metric,m0,m1,decision;\
    metric = ((Branchtab27_1[i] ^ sym1) + (Branchtab27_2[i] ^ sym2) + 1)/2;\
    m0 = vp->old_metrics[i] + metric;\
    m1 = vp->old_metrics[i+32] + (15 - metric);\
    decision = (m0-m1) >= 0;\
    vp->new_metrics[2*i] = decision ? m1 : m0;\
    vp->dp->w[i/16] |= decision << ((2*i)&31);\
    m0 -= (metric+metric-15);\
    m1 += (metric+metric-15);\
    decision = (m0-m1) >= 0;\
    vp->new_metrics[2*i+1] = decision ? m1 : m0;\
    vp->dp->w[i/16] |= decision << ((2*i+1)&31);\
}
#define EXTRACT_DECISION(d,state) (((d)->w[(state)/32] >> ((state)%32)) & 1)

static unsigned char Branchtab27_1[32];
static unsigned char Branchtab27_2[32];

// AMSAT Phase 3 epoch of 1 Jan 1978 00:00:00 UTC in UNIX time
#define EPOCH ((time_t)252460800)

const char Header[257] =
{
  "A  HI, THIS IS AMSAT OSCAR-40       yyyy-mm-dd  hh:mm:ss  #nnnn "
  "+--------------------------------------------------------------+"
  "|             RECONSTRUCTED FROM DECODED FEC BLOCK             |"
  "+--------------------------------------------------------------+"
};

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CFECDecode::CFECDecode()
{
	m_RS.alpha_to = NULL;
	m_RS.genpoly = NULL;
	m_RS.index_of = NULL;
	m_V27init = FALSE;
	m_pVs = NULL;
	Init();
}

CFECDecode::~CFECDecode()
{
	if(m_RS.alpha_to)
		delete m_RS.alpha_to;
	if(m_RS.genpoly)
		delete m_RS.genpoly;
	if(m_RS.index_of)
		delete m_RS.index_of;
	DeleteViterbi27(m_pVs);

}

void CFECDecode::Init()
{
int sr,i,j;
	//Generate Sync Vector
	sr = 0x7f;
	for(i=0; i<COLUMNS; i++)
	{
		m_SyncVector[i] = (sr & 64) ? 1 : 0;
		sr = (sr << 1) | parityb(sr & PNPOLY);
	}
	//Generate Scrambler Sequence
	sr = 0xff;
	for(i=0; i<sizeof(m_Scrambler); i++)
	{
		m_Scrambler[i] = sr;
		for(j=0; j<8; j++)
			sr = (sr << 1) | parity(sr & SCRAMBLER_POLY);
	}
	//Init Reed Solomon Decoder
	InitRS(8,0x187,112,11,32,RSPAD); // CCSDS standard
	//Init Viterbi Decoder
	if(!m_V27init)
		m_pVs = CreateViterbi27(SYMPBLOCK/2);
	m_InPtr = 0;
	m_SymCounter = 0;
	m_AmplitudeAve = 0.;
	for( i=0; i<SYMBUFSIZE; i++)
		m_SymbBuf[i] = 0;
	m_InPtr = 0;
}

////////////////////////////////////////////////////////////////////
//       Process each symbol as it comes in at 400Hz rate
////////////////////////////////////////////////////////////////////
BOOL CFECDecode::ProcessFECSymbol(int symb, ECstats *pStats)
{
int i,j;
int BlkStart;
int scale;
int col,row;
int syms[2];
double CorAmp = 0.;
double FrameAmp = 0.;
	pStats->RS0errors = -1;
	pStats->RS1errors = -1;
	////////////////////////////////////////////////////
	///////////   Look for Sync Vector    //////////////
	////////////////////////////////////////////////////
	// keep running average of scaled symbol amplitudes
	m_AmplitudeAve = (1.0-1./400)*m_AmplitudeAve + (1.0/(5500.0*400.0))*abs(symb);	
	m_SymbBuf[m_InPtr] = symb;	//place new symbol in circular buffer
	BlkStart = (m_InPtr - SYMPBLOCK) & BUFPTRMOD;	//calc possible block start
	m_InPtr = (m_InPtr + 1) & BUFPTRMOD;	//inc input pointer dealing with wrap around
	//Look for sync vector by summing all the sync bit position energies
	i = BlkStart;
	for(col=0; col<COLUMNS; col++)
	{
		double fpsym = m_SymbBuf[i];
		CorAmp += (double)( m_SyncVector[col] ? fpsym : -fpsym );
		i = (i + ROWS) & BUFPTRMOD;
		FrameAmp += (fpsym*fpsym);
	}
	if(	m_SymCounter <= SYMPBLOCK )
		m_SymCounter++;		//searching for sync
//DebugFP1 = (m_AmplitudeAve);
//DebugFP2 = CorAmp/sqrt(FrameAmp);
	if( CorAmp < 4.1*sqrt(FrameAmp) ) //see if sync energy exceeds valid threshold
		return FALSE;			//return if sync vector not reached or found
	////////////////////////////////////////////////////
	///////////   Here with New Block     //////////////
	////////////////////////////////////////////////////
	if(	m_SymCounter < SYMPBLOCK/10)
	{
		return FALSE;	//false sync if happened too soon after last one
	}
//StartPerformance();
	m_SymCounter = 0;
	scale = (int)(32768.0/m_AmplitudeAve);	//scale value for symbol to fit in 8 bits
	ResetViterbi27(m_pVs, 0);	//Reset Viterbi decoder state
	pStats->ok = FALSE;
	////////////////////////////////////////////////////
	////   De-Interleave and run Viterbi Decoder ///////
	////////////////////////////////////////////////////
	int cc = 0;
	for(row=1; row<ROWS; row++) // Skip row 0, which is the sync vector
	{
		for(col=0; col<COLUMNS; col++)
		{
			i = (BlkStart + col*ROWS + row) & BUFPTRMOD;
			int s = ( (scale*m_SymbBuf[i])>>25 );
			syms[cc] = ClipToNibble(8 + s );	//scale symbol to 4 bits
			if(cc++ == 1)
			{
				int d;
				d = UpdateViterbi27(m_pVs,15-syms[1],syms[0]);
				cc = 0;
			}
		}
	}
	ChainbackViterbi27(	m_pVs,	m_VData, sizeof(m_VData)*8, 0);

	///////////////////////////////////////////////////////////////
	//  De-Scramble and Interleave into Reed Solomon Code blocks //
	///////////////////////////////////////////////////////////////
	unsigned char rsBlocks[RSBLOCKS][RSTOT];
	i = 0;
	memset(rsBlocks,0,sizeof(rsBlocks));
	for(col=0; col<RSTOT; col++)
	{
		for(row=0; row<RSBLOCKS; row++)
		{
			rsBlocks[row][col] = m_VData[i] ^ m_Scrambler[i];
			i++;
		}
	}
	///////////////////////////////////////////////////////////////
	///////  Run Reed Solomon Decoder on the two Code blocks //////
	///////////////////////////////////////////////////////////////
	int	rs_failures = 0;
	for(row=0; row<RSBLOCKS; row++)
	{
		m_rsErrors[row] = DecodeRS8(rsBlocks[row], NULL, 0, RSPAD);
		rs_failures += (m_rsErrors[row] == -1);
	}

	if(rs_failures == RSBLOCKS)
	{ // Nothing decoded, so we can't even be sure a frame was there
		TRACE0("Nothing decoded\n");
//StopPerformance();
		return FALSE;
	}
	// At least a partial decode, so we know we have correct sync

#if 0
	TRACE0("RS byte corrections:");
	for(row=0;row<RSBLOCKS;row++)
	{
		if(m_rsErrors[row] != -1)
			TRACE(" %d",m_rsErrors[row]);
		else
			TRACE0(" ?");
	}
	TRACE(" - %s\n",rs_failures ? "Bad Frame":"Good Frame");
#endif
	pStats->RS0errors = m_rsErrors[0];
	pStats->RS1errors = m_rsErrors[1];
	if(rs_failures != 0)
	{
		// The block didn't decode completely, but at least we know a frame
		// was really there
//StopPerformance();
		return TRUE;
	}
	// Decoded perfectly, no failures
	// Deinterleave data from RS codewords
	j = 0;
	for(col=0;col<RSDATA;col++)
	{
		for(row=0;row<RSBLOCKS;row++)
		{
			m_Data[j++] = rsBlocks[row][col];
		}
	}
	pStats->ok = TRUE;
	//Re-encode and count how many symbols were corrected
	int Recoded[5200];
	int errors = 0;
	m_FECencode.EncodeFrame( Recoded ,m_Data);
	for(i=0; i<SYMPBLOCK; i++)
	{
		j = (BlkStart+i) & BUFPTRMOD;
	    if( Recoded[i] != (m_SymbBuf[j] > 0) )
	      errors++;
	}
	pStats->SYMerrors = errors;
//StopPerformance();
	return TRUE;
}


//////////////////////////////////////////////////////////////////////////
// Initialize a Reed-Solomon codec
// symsize = symbol size, bits (1-8)
// gfpoly = Field generator polynomial coefficients
// fcr = first root of RS code generator polynomial, index form
// prim = primitive element to generate polynomial roots
// nroots = RS code generator polynomial degree (number of roots)
// pad = padding bytes at front of shortened block
//				Copyright 1997 Phil Karn, KA9Q
//////////////////////////////////////////////////////////////////////////
int CFECDecode::InitRS(int symsize,int gfpoly,int fcr,int prim, int nroots,int pad)
{
int i, j, sr,root,iprim;

	// Check parameter ranges
	if(symsize < 0 || symsize > 8*sizeof(unsigned char))
		return NULL; // Need version with ints rather than chars

	if(fcr < 0 || fcr >= (1<<symsize))
		return NULL;
	if(prim <= 0 || prim >= (1<<symsize))
		return NULL;
	if(nroots < 0 || nroots >= (1<<symsize))
		return NULL; // Can't have more roots than symbol values!
	if(pad < 0 || pad >= ((1<<symsize) -1 - nroots))
		return NULL; // Too much padding

	m_RS.mm = symsize;
	m_RS.nn = (1<<symsize)-1;
	m_RS.pad = pad;

	//deallocate memory if already allocated
	if(m_RS.alpha_to)
		delete m_RS.alpha_to;
	if(m_RS.genpoly)
		delete m_RS.genpoly;
	if(m_RS.index_of)
		delete m_RS.index_of;

	m_RS.alpha_to = new unsigned char[m_RS.nn+1];
	if(m_RS.alpha_to == NULL)
	{
		return NULL;
	}
	m_RS.index_of = new unsigned char[m_RS.nn+1];
	if(m_RS.index_of == NULL)
	{
		delete m_RS.alpha_to;
		return NULL;
	}

	//Generate Galois field lookup tables
	m_RS.index_of[0] = NN; // log(zero) = -inf
	m_RS.alpha_to[NN] = 0; // alpha**-inf = 0
	sr = 1;
	for(i=0;i<m_RS.nn;i++)
	{
		m_RS.index_of[sr] = i;
		m_RS.alpha_to[i] = sr;
		sr <<= 1;
		if(sr & (1<<symsize))
			sr ^= gfpoly;
		sr &= m_RS.nn;
	}
	if(sr != 1)
	{  // field generator polynomial is not primitive! 
		delete m_RS.alpha_to;
		delete m_RS.index_of;
		return NULL;
	}

	// Form RS code generator polynomial from its roots 
	m_RS.genpoly = new unsigned char[nroots+1];
	if(m_RS.genpoly == NULL)
	{
		delete m_RS.alpha_to;
		delete m_RS.index_of;
		return NULL;
	}
	m_RS.fcr = fcr;
	m_RS.prim = prim;
	m_RS.nroots = nroots;

	// Find prim-th root of 1, used in decoding 
	for(iprim=1;(iprim % prim) != 0;iprim += m_RS.nn)
	{ }
	m_RS.iprim = iprim / prim;

	m_RS.genpoly[0] = 1;
	for (i = 0,root=fcr*prim; i < nroots; i++,root += prim)
	{
		m_RS.genpoly[i+1] = 1;

	    // Multiply m_RS.genpoly[] by  @**(root + x)
		for (j = i; j > 0; j--)
		{
			if (m_RS.genpoly[j] != 0)
				m_RS.genpoly[j] = m_RS.genpoly[j-1] ^ 
						m_RS.alpha_to[Modnn(m_RS.index_of[m_RS.genpoly[j]] + root)];
			else
				m_RS.genpoly[j] = m_RS.genpoly[j-1];
		}
		// m_RS.genpoly[0] can never be zero
		m_RS.genpoly[0] = m_RS.alpha_to[Modnn(m_RS.index_of[m_RS.genpoly[0]] + root)];
	}
	// convert m_RS.genpoly[] to index form for quicker encoding 
	for (i = 0; i <= nroots; i++)
		m_RS.genpoly[i] = m_RS.index_of[m_RS.genpoly[i]];
	return NULL;
}

//Helper function for mod calc
int CFECDecode::Modnn(int x)
{
	while (x >= m_RS.nn)
	{
		x -= m_RS.nn;
		x = (x >> m_RS.mm) + (x & m_RS.nn);
	}
	return x;
}

//////////////////////////////////////////////////////////////////////
//================  Veterbi27 Decoder routines  ======================
//				Copyright 1997 Phil Karn, KA9Q
//////////////////////////////////////////////////////////////////////
v27* CFECDecode::CreateViterbi27(int len)
{
void *blk;
struct v27 *vp;
int state;
    // Initialize metric tables
	if(!m_V27init)
	{
		for(state=0;state < 32;state++)
		{
			Branchtab27_1[state] = parity((2*state) & V27POLYA) ? 15:0;
			Branchtab27_2[state] = parity((2*state) & V27POLYB) ? 15:0;
		}
		m_V27init = 1;
	}
	// Malloc only guarantees 8-byte alignment, but we want to ensure that
	// the path metric arrays are on 32-byte boundaries. At least 16-byte
	// alignment is mandatory in the SSE2 version, but the Pentium III
	// cache line size is 32 bytes
	blk = malloc(sizeof(struct v27)+32);
	if((int)blk & 31)
	{	// Not on 32-byte boundary; shift up
		vp = (struct v27 *)(((int)blk + 32) & ~31);
	}
	else
	{
		vp = (struct v27 *)blk;
	}
	vp->alloc_blk = blk; 
	// Record original pointer from malloc for use by free()
	// The decisions only need be 32-bit aligned
	vp->dp = vp->decisions = (decision_t*)malloc((len+6)*8);
	vp->old_metrics = vp->metrics1;
	vp->new_metrics = vp->metrics2;
	return vp;
}

// Reset Viterbi decoder for start of new frame
int CFECDecode::ResetViterbi27(v27 *p,int starting_state)
{
struct v27 *vp = p;
	memset(vp->metrics1,60,64);
	vp->old_metrics = vp->metrics1;
	vp->new_metrics = vp->metrics2;
	vp->dp = vp->decisions;
	vp->old_metrics[starting_state & 63] = 0; // Bias known start state
	return 0;
}

// Delete instance of a Viterbi decoder 
void CFECDecode::DeleteViterbi27(v27 *p)
{
struct v27 *vp = p;
	if(vp != NULL)
	{
		free(vp->decisions);
		free(vp->alloc_blk);
	}
}

// Process two new symbols
int CFECDecode::UpdateViterbi27(v27 *p,unsigned char sym1,unsigned char sym2)
{
struct v27 *vp = p;
unsigned char *tmp;
int normalize = 0;

	vp->dp->w[0] = vp->dp->w[1] = 0;
	BFLY(0);
	BFLY(1);
	BFLY(2);
	BFLY(3);
	BFLY(4);
	BFLY(5);
	BFLY(6);
	BFLY(7);
	BFLY(8);
	BFLY(9);
	BFLY(10);
	BFLY(11);
	BFLY(12);
	BFLY(13);
	BFLY(14);
	BFLY(15);
	BFLY(16);
	BFLY(17);
	BFLY(18);
	BFLY(19);
	BFLY(20);
	BFLY(21);
	BFLY(22);
	BFLY(23);
	BFLY(24);
	BFLY(25);
	BFLY(26);
	BFLY(27);
	BFLY(28);
	BFLY(29);
	BFLY(30);
	BFLY(31);

	// Renormalize metrics
	if(vp->new_metrics[0] > 150)
	{
		int i;
		unsigned char minmetric = 255;
		for(i=0;i<64;i++)
			if(vp->new_metrics[i] < minmetric)
		minmetric = vp->new_metrics[i];
		for(i=0;i<64;i++)
			vp->new_metrics[i] -= minmetric;
		normalize = minmetric;
	}

	vp->dp++;
	tmp = vp->old_metrics;
	vp->old_metrics = vp->new_metrics;
	vp->new_metrics = tmp;

	return normalize;
}

// Do Viterbi chainback
int CFECDecode::ChainbackViterbi27(
	v27 *p,
	unsigned char *data, // Decoded output data
	unsigned int nbits, // Number of data bits
	unsigned int endstate) // Terminal encoder state
{
struct v27 *vp = p;
int k;
	decision_t *decisions = (decision_t *)(vp->decisions);

	// Make room beyond the end of the encoder register so we can
	// accumulate a full byte of decoded data
	endstate %= 64;
	endstate <<= 2;
	decisions += 6; // Look past tail

	while(nbits-- != 0)
	{
		k = EXTRACT_DECISION(&decisions[nbits],endstate >> 2);
		// The store into data[] only needs to be done every 8 bits.
		// But this avoids a conditional branch, and the writes will
		// combine in the cache anyway
		data[nbits>>3] = endstate = (endstate >> 1) | (k << 7);
	}
	return 0;
}

////////////////////////////////////////////////////////////////////
///////////////////// Reed Solomon Decoder  ////////////////////////
//				Copyright 1997 Phil Karn, KA9Q
////////////////////////////////////////////////////////////////////
int CFECDecode::DecodeRS8(unsigned char *data, int *eras_pos, int no_eras,int pad)
{
int deg_lambda, el, deg_omega;
int i, j, r,k;
unsigned char u,q,tmp,num1,num2,den,discr_r;
unsigned char lambda[NROOTS+1], s[NROOTS];	/* Err+Eras Locator poly
			 * and syndrome poly */
unsigned char b[NROOTS+1], t[NROOTS+1], omega[NROOTS+1];
unsigned char root[NROOTS], reg[NROOTS+1], loc[NROOTS];
int syn_error, count;

// Check pad parameter for validity
	if(pad < 0 || pad >= NN)
		return -1;

// form the syndromes; i.e., evaluate data(x) at roots of g(x)
	for(i=0;i<NROOTS;i++)
		s[i] = data[0];

	for(j=1;j<NN-PAD;j++)
	{
		for(i=0;i<NROOTS;i++)
		{
			if(s[i] == 0)
			{
				s[i] = data[j];
			}
			else
			{
				s[i] = data[j] ^ ALPHA_TO[Modnn(INDEX_OF[s[i]] + (FCR+i)*PRIM)];
			}
		}
	}

	// Convert syndromes to index form, checking for nonzero condition
	syn_error = 0;
	for(i=0;i<NROOTS;i++)
	{
		syn_error |= s[i];
		s[i] = INDEX_OF[s[i]];
	}

	if(!syn_error)
	{
		// if syndrome is zero, data[] is a codeword and there are no
		// errors to correct. So return data[] unmodified
		count = 0;
		return count;
	}
	memset( &lambda[1], 0, NROOTS*sizeof(lambda[0]) );
	lambda[0] = 1;

	if(no_eras > 0)
	{
		// Init lambda to be the erasure locator polynomial
		lambda[1] = ALPHA_TO[Modnn(PRIM*(NN-1-eras_pos[0]))];
		for(i = 1; i < no_eras; i++)
		{
			u = Modnn(PRIM*(NN-1-eras_pos[i]));
			for(j = i+1; j > 0; j--)
			{
				tmp = INDEX_OF[lambda[j - 1]];
				if(tmp != A0)
					lambda[j] ^= ALPHA_TO[Modnn(u + tmp)];
			}
		}

#if 0
// Test code that verifies the erasure locator polynomial just constructed
// Needed only for decoder debugging.
		// find roots of the erasure location polynomial
		for(i=1;i<=no_eras;i++)
			reg[i] = INDEX_OF[lambda[i]];
		count = 0;
		for (i = 1,k=IPRIM-1; i <= NN; i++,k = Modnn(k+IPRIM))
		{
			q = 1;
			for(j = 1; j <= no_eras; j++)
				if(reg[j] != A0)
				{
					reg[j] = Modnn(reg[j] + j);
					q ^= ALPHA_TO[reg[j]];
				}
			if (q != 0)
				continue;
		// store root and error location number indices
		root[count] = i;
		loc[count] = k;
		count++;
		}
		if (count != no_eras)
		{
			TRACE("count = %d no_eras = %d\n lambda(x) is WRONG\n",count,no_eras);
			count = -1;
			return count;
		}
#if 0
		TRACE("\n Erasure positions as determined by roots of Eras Loc Poly:\n");
		for (i = 0; i < count; i++)
			TRACE("%d ", loc[i]);
		TRACE("\n");
#endif
#endif
	}
	for(i=0;i<NROOTS+1;i++)
		b[i] = INDEX_OF[lambda[i]];

	// Begin Berlekamp-Massey algorithm to determine error+erasure
	// locator polynomial
	r = no_eras;
	el = no_eras;
	while (++r <= NROOTS)
	{	// r is the step number
		// Compute discrepancy at the r-th step in poly-form
		discr_r = 0;
		for (i = 0; i < r; i++)
		{
			if( (lambda[i] != 0) && (s[r-i-1] != A0) )
			{
				discr_r ^= ALPHA_TO[Modnn(INDEX_OF[lambda[i]] + s[r-i-1])];
			}
		}
		discr_r = INDEX_OF[discr_r];	// Index form
		if (discr_r == A0)
		{
			// 2 lines below: B(x) <-- x*B(x)
			memmove(&b[1],b,NROOTS*sizeof(b[0]));
			b[0] = A0;
		}
		else
		{
			// 8 lines below: T(x) <-- lambda(x) - discr_r*x*b(x)
			t[0] = lambda[0];
			for (i = 0 ; i < NROOTS; i++)
			{
				if(b[i] != A0)
					t[i+1] = lambda[i+1] ^ ALPHA_TO[Modnn(discr_r + b[i])];
				else
					t[i+1] = lambda[i+1];
			}
			if (2 * el <= r + no_eras - 1)
			{
				el = r + no_eras - el;
				// 2 lines below: B(x) <-- inv(discr_r)
				// lambda(x)
				for (i = 0; i <= NROOTS; i++)
					b[i] = (lambda[i] == 0) ? A0 : Modnn(INDEX_OF[lambda[i]] - discr_r + NN);
			}
			else
			{
				// 2 lines below: B(x) <-- x*B(x)
				memmove(&b[1],b,NROOTS*sizeof(b[0]));
				b[0] = A0;
			}
			memcpy(lambda,t,(NROOTS+1)*sizeof(t[0]));
		}
	}

	// Convert lambda to index form and compute deg(lambda(x))
	deg_lambda = 0;
	for(i=0;i<NROOTS+1;i++)
	{
		lambda[i] = INDEX_OF[lambda[i]];
		if(lambda[i] != A0)
			deg_lambda = i;
	}
	// Find roots of the error+erasure locator polynomial by Chien search
	memcpy(&reg[1],&lambda[1],NROOTS*sizeof(reg[0]));
	count = 0;		// Number of roots of lambda(x)
	for(i = 1,k=IPRIM-1; i <= NN; i++,k = Modnn(k+IPRIM))
	{
		q = 1; // lambda[0] is always 0
		for (j = deg_lambda; j > 0; j--)
		{
			if(reg[j] != A0)
			{
				reg[j] = Modnn(reg[j] + j);
				q ^= ALPHA_TO[reg[j]];
			}
		}
		if (q != 0)
			continue; /* Not a root */
		// store root (index-form) and error location number
#if 0
	TRACE("count %d root %d loc %d\n",count,i,k);
#endif
		root[count] = i;
		loc[count] = k;
		// If we've already found max possible roots,
		// abort the search to save time
		if(++count == deg_lambda)
			break;
	}
	if (deg_lambda != count)
	{
		// deg(lambda) unequal to number of roots => uncorrectable
		// error detected
		count = -1;
		return count;
	}
	// Compute err+eras evaluator poly omega(x) = s(x)*lambda(x) (modulo
	// x**NROOTS). in index form. Also find deg(omega).
	deg_omega = deg_lambda-1;
	for (i = 0; i <= deg_omega;i++)
	{
		tmp = 0;
		for(j=i;j >= 0; j--)
		{
			if((s[i - j] != A0) && (lambda[j] != A0))
				tmp ^= ALPHA_TO[Modnn(s[i - j] + lambda[j])];
		}
		omega[i] = INDEX_OF[tmp];
	}
	// Compute error values in poly-form. num1 = omega(inv(X(l))), num2 =
	// inv(X(l))**(FCR-1) and den = lambda_pr(inv(X(l))) all in poly-form
	for (j = count-1; j >=0; j--)
	{
		num1 = 0;
		for(i = deg_omega; i >= 0; i--)
		{
			if(omega[i] != A0)
				num1  ^= ALPHA_TO[Modnn(omega[i] + i * root[j])];
		}
		num2 = ALPHA_TO[Modnn(root[j] * (FCR - 1) + NN)];
		den = 0;

		// lambda[i+1] for i even is the formal derivative lambda_pr of lambda[i]
		for (i = min(deg_lambda,NROOTS-1) & ~1; i >= 0; i -=2)
		{
			if(lambda[i+1] != A0)
				den ^= ALPHA_TO[Modnn(lambda[i+1] + i * root[j])];
		}
#if 0
if (den == 0)
{
	TRACE("\n ERROR: denominator = 0\n");
	count = -1;
	return count;
}
#endif
		// Apply error to data
		if (num1 != 0 && loc[j] >= PAD)
		{
			data[loc[j]-PAD] ^= ALPHA_TO[Modnn(INDEX_OF[num1] + INDEX_OF[num2] + NN - INDEX_OF[den])];
		}
	}
	if(eras_pos != NULL)
	{
		for(i=0;i<count;i++)
			eras_pos[i] = loc[i];
	}
	return count;
}

/////////////////////////////////////////////////////////////////////
// Must reconstruct a full AO40 Header from just the bottom 256 byte
// data section by adding header information
/////////////////////////////////////////////////////////////////////
void CFECDecode::GetDataBlock(BOOL GoodBlock, unsigned char *pData)
{
time_t t;
int days,hours,mins,secs,cmdno;
struct tm *tm;
	memcpy( pData, Header, sizeof(Header) );
	memcpy( pData+256, m_Data, sizeof(m_Data) );
	if(GoodBlock)
	{
		// Extract command number
		cmdno = 256*pData[0x1e1] + pData[0x1e0];
    
		// Extract time
		days = 256*pData[0x1ad] + pData[0x1ac];
		hours =  pData[0x1ab];
		mins = pData[0x1aa];
		secs = pData[0x1a9];
		t = EPOCH + 86400*days + 3600*hours + 60*mins + secs;
		tm = gmtime(&t);
		if(tm==0)
			return;
		sprintf((char*)&pData[36],"%4d-%02d-%02d  %02d:%02d:%02d  #%04X",
			tm->tm_year+1900,tm->tm_mon+1,tm->tm_mday,tm->tm_hour,
			tm->tm_min,tm->tm_sec,cmdno);
		pData[63] = ' '; // Overwrite null termination from sprintf
	}
}