// FECDecode.h: interface for the CFECDecode class.
//
////////////////////////////////////////////////////////////////////
//				Copyright 1997 Phil Karn, KA9Q
////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_FECDECODE_H__1203287F_86F7_4A92_A908_22C06EE68CBB__INCLUDED_)
#define AFX_FECDECODE_H__1203287F_86F7_4A92_A908_22C06EE68CBB__INCLUDED_

#include "Ao40Fec.h"

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#define BLOCKSIZE  256          /* Data bytes per frame */
#define PNPOLY     0x48         /* Sync vector PN polynomial */
#define SCRAMBLER_POLY 0x95     /* Scrambler polynomial */
#define ICRATE     2            /* Inverse of convolutional encoder rate */
#define CK         7            /* Convolutional encoder constraint length */
#define ROWS       80           /* Block interleaver rows */
#define COLUMNS    65           /* Block interleaver columns */
#define RSBLOCKS   2            /* Reed-Solomon blocks per frame */

/* Derived and fixed constants - don't change */
#define SYMPBLOCK (ROWS*COLUMNS)      /* Encoded symbols per block */

#define NN 255    /* Total bytes per unshortened RS code block */
#define KK 223    /* Data bytes per unshortened RS code block */
#define RSDATA    (BLOCKSIZE/RSBLOCKS)    /* Data bytes per shortened RS code block */
#define RSPAD     (KK-RSDATA)             /* Padding bytes per RS code block */
#define RSTOT     (NN-KK+RSDATA)          /* Total bytes (data + parity) per shortened RS code block */

#define SYMBUFSIZE 8192		//must be 8192 for pointer wrapping to work
#define BUFPTRMOD 0x1FFF	//Modulus mask for queue pointer wrap around

typedef struct
{
	int RS0errors;
	int RS1errors;
	int SYMerrors;
	BOOL ok;
}ECstats;

typedef union { LONGLONG p; unsigned long w[2]; } decision_t;
struct v27
{
	unsigned char metrics1[64]; /* path metric buffer 1 */
	unsigned char metrics2[64]; /* path metric buffer 2 */
	decision_t *dp;              /* Pointer to decision output for current bit */
	unsigned char *old_metrics,*new_metrics; /* Pointers to path metrics, swapped on every bit */
	decision_t *decisions;       /* Beginning of decisions for block */
	void *alloc_blk;             /* Return value from malloc */
};

class CFECDecode  
{
public:
	CFECDecode();
	virtual ~CFECDecode();
	void GetDataBlock( BOOL GoodBlk, unsigned char* pData );
	BOOL ProcessFECSymbol(int symb, ECstats* pStats);
	void Init();
private:
	int InitRS(int symsize,int gfpoly,int fcr,int prim, int nroots,int pad);
	int Modnn(int x);
	v27* CreateViterbi27(int len);
	int ResetViterbi27(v27 *p,int starting_state);
	void DeleteViterbi27(v27 *p);
	int UpdateViterbi27(v27 *p,unsigned char sym1,unsigned char sym2);
	int ChainbackViterbi27(	v27 *p,	unsigned char *data, unsigned int nbits, unsigned int endstate);
	int DecodeRS8(unsigned char *data, int *eras_pos, int no_eras,int pad);
	
	BOOL m_V27init;
	int m_SyncVector[COLUMNS];
	unsigned char m_Scrambler[BLOCKSIZE+RSBLOCKS*(NN-KK)];
	int m_SymbBuf[SYMBUFSIZE];	//circular modulo 8192 buffer for symbols
	unsigned char m_VData[RSBLOCKS*RSTOT];
	unsigned char m_Data[BLOCKSIZE];
	int m_rsErrors[RSBLOCKS];
	int m_InPtr;
	int m_SymCounter;
	double m_AmplitudeAve;
	v27* m_pVs;
	CAo40Fec m_FECencode;

	inline int update(int x)
	{
		if(x == SYMPBLOCK-1)
			return -1; /* Was last symbol, can't update */
		x += ROWS;
		if(x >= SYMPBLOCK)
			x -= (SYMPBLOCK-1);
		return x;
	}
	inline int parityb(unsigned char x)
	{
		__asm
		{
			test x,0xFF
			setpo x
		}
		return (int)x;
	}
	inline int parity(int x)
	{	//Fold down to one byte
		x ^= (x >> 16);
		x ^= (x >> 8);
		return parityb(x);
	}
	inline int mod255(int x)
	{
		while (x >= 255)
		{
			x -= 255;
			x = (x >> 8) + (x & 255);
		}
		return x;
	}

	inline int ClipToNibble(int x)
	{
		return x < 0 ? 0 : (x > 15 ? 15 : x);
	}

	struct rs	// Reed-Solomon codec control block
	{
		int mm;              /* Bits per symbol */
		int nn;              /* Symbols per block (= (1<<mm)-1) */
		unsigned char *alpha_to;     /* log lookup table */
		unsigned char *index_of;     /* Antilog lookup table */
		unsigned char *genpoly;      /* Generator polynomial */
		int nroots;     /* Number of generator roots = number of parity symbols */
		int fcr;        /* First consecutive root, index form */
		int prim;       /* Primitive element, index form */
		int iprim;      /* prim-th root of 1, index form */
		int pad;        /* Padding bytes in shortened block */
	}m_RS;

};

#endif // !defined(AFX_FECDECODE_H__1203287F_86F7_4A92_A908_22C06EE68CBB__INCLUDED_)
