// MainFrm.cpp : implementation of the CMainFrame class
//
//////////////////////////////////////////////////////////////////////
// Copyright 2000.    Moe Wheatley AE4JY  <ae4jy@qsl.net>
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "AO40Rcv.h"

#include "MainFrm.h"
#include "time.h"
#include "DispDlg.h"
#include "InSelect.h"
#include "RigDlg.h"
#include "AO40RcvDoc.h"
#include "AO40RcvView.h"
#include "MainFrm.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define ID_TIMER 5

/////////////////////////////////////////////////////////////////////////////
// CMainFrame

IMPLEMENT_DYNCREATE(CMainFrame, CFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CFrameWnd)
	//{{AFX_MSG_MAP(CMainFrame)
	ON_WM_CREATE()
	ON_WM_CLOSE()
	ON_COMMAND(ID_VIEW_STAYONTOP, OnViewStayontop)
	ON_UPDATE_COMMAND_UI(ID_VIEW_STAYONTOP, OnUpdateViewStayontop)
	ON_WM_TIMER()
	ON_COMMAND(ID_INPUT_LEVEL_ADJ, OnInputLevelAdj)
	ON_COMMAND(IDM_OUTPUT_SETTINGS, OnOutputSettings)
	ON_COMMAND(IDM_RIG_CTRL, OnRigCtrl)
	ON_COMMAND(IDM_INPUT_SELECT, OnInputSelect)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

static UINT indicators[] =
{
	IDS_STATUSBARX1,
	IDS_STATUSBARX2,
	ID_SEPARATOR,
};

/////////////////////////////////////////////////////////////////////////////
// CMainFrame construction/destruction

CMainFrame::CMainFrame()
{
	m_wp.length = 0;
	m_StayOnTop = FALSE;
	
}

CMainFrame::~CMainFrame()
{
}

////////////////////////////////////////////////////////////////////////
//  Called to create the main window frame
////////////////////////////////////////////////////////////////////////
int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	if (!m_wndToolBar.CreateEx(this, TBSTYLE_FLAT, WS_CHILD /*| WS_VISIBLE */| CBRS_TOP
		| CBRS_GRIPPER | CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC) ||
		!m_wndToolBar.LoadToolBar(IDR_MAINFRAME))
	{
		TRACE0("Failed to create toolbar\n");
		return -1;      // fail to create
	}

	if (!m_wndStatusBar.Create(this) ||
		!m_wndStatusBar.SetIndicators(indicators,
		  sizeof(indicators)/sizeof(UINT)))
	{
		TRACE0("Failed to create status bar\n");
		return -1;      // fail to create
	}
	m_wp.length = sizeof(WINDOWPLACEMENT);
// get screen size settings if they are saved in registry
	if( !LoadWindowPlacement( &m_wp ) )
	{
		// set default sizes if none are loaded from the registry
		m_wp.length = 0;
		m_StayOnTop = FALSE;
	}

	LoadBarState("ControlBars\\State");	//restore status/toolbar states
	m_TimerID = SetTimer( ID_TIMER, 1000, NULL);	//start up 1 sec status timer
	if( m_TimerID == 0)
		AfxMessageBox( "Can't Start Status Timer",MB_OK );
	return 0;
}

////////////////////////////////////////////////////////////////////////
//  Called to modify the window frame
////////////////////////////////////////////////////////////////////////
BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
{
	if( !CFrameWnd::PreCreateWindow(cs) )
		return FALSE;
	cs.style &= ~FWS_ADDTOTITLE;
	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CMainFrame diagnostics

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
	CFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
	CFrameWnd::Dump(dc);
}

#endif //_DEBUG

///////////////////////////////////////////////////////////////
// get registry screen size settings
///////////////////////////////////////////////////////////////
BOOL CMainFrame::LoadWindowPlacement(LPWINDOWPLACEMENT pwp)
{
	CString strBuffer = AfxGetApp ()->GetProfileString ("WindowPos", "WindowPos");
	if (strBuffer.IsEmpty ())
		return FALSE;
	int cRead = _stscanf (strBuffer, "%i:%i:%i:%i:%i:%i:%i:%i:%i:%i",
			&pwp->flags, &pwp->showCmd,
			&pwp->ptMinPosition.x, &pwp->ptMinPosition.y,
			&pwp->ptMaxPosition.x, &pwp->ptMaxPosition.y,
			&pwp->rcNormalPosition.left, &pwp->rcNormalPosition.top,
			&pwp->rcNormalPosition.right, &pwp->rcNormalPosition.bottom);

	if(pwp->flags & 0x8000)
		m_StayOnTop = TRUE;
	else
		m_StayOnTop = FALSE;
	pwp->flags &= ~0x8000;

	if (cRead != 10)
		return FALSE;
	return TRUE;

}

///////////////////////////////////////////////////////////////
// save screen size settings to registry
///////////////////////////////////////////////////////////////
void CMainFrame::SaveWindowPlacement(LPWINDOWPLACEMENT pwp)
{
	// SaveWindowPlacement
	if(m_StayOnTop)
		pwp->flags |= 0x8000;
	else
		pwp->flags &= ~0x8000;
	CString strBuffer;
	strBuffer.Format ("%i:%i:%i:%i:%i:%i:%i:%i:%i:%i",
		pwp->flags, pwp->showCmd,
		pwp->ptMinPosition.x, pwp->ptMinPosition.y,
		pwp->ptMaxPosition.x, pwp->ptMaxPosition.y,
		pwp->rcNormalPosition.left, pwp->rcNormalPosition.top,
		pwp->rcNormalPosition.right, pwp->rcNormalPosition.bottom);

	AfxGetApp()->WriteProfileString("WindowPos", "WindowPos", strBuffer);
}

/////////////////////////////////////////////////////////////////////////////
// CMainFrame message handlers


////////////////////////////////////////////////////////////////////////
//  Called on program close to clean up
////////////////////////////////////////////////////////////////////////
void CMainFrame::OnClose() 
{
WINDOWPLACEMENT wp;
	if( m_TimerID != 0 )			// stop timer if running
	{
		KillTimer( m_TimerID );
		m_TimerID = 0;
	}
	wp.length = sizeof(WINDOWPLACEMENT);
	if (GetWindowPlacement(&wp) )
	{
		if( IsZoomed() )
			wp.flags |= WPF_RESTORETOMAXIMIZED;
		SaveWindowPlacement( &wp );
	}

	SaveBarState("ControlBars\\State");	
	CFrameWnd::OnClose();
}

////////////////////////////////////////////////////////////////////////
//  Called to deal with frame state
////////////////////////////////////////////////////////////////////////
void CMainFrame::ActivateFrame(int nCmdShow) 
{
	if(	m_wp.length != 0)	//if structure has been filled in
	{
		nCmdShow= m_wp.showCmd;
		AfxGetApp ()->m_nCmdShow = nCmdShow;
		SetWindowPlacement(&m_wp);
	}
	else
		AfxGetApp ()->m_nCmdShow = nCmdShow;	
	CFrameWnd::ActivateFrame(nCmdShow);
}

////////////////////////////////////////////////////////////////////////
//  Routines to deal with stay on top function
////////////////////////////////////////////////////////////////////////
void CMainFrame::OnViewStayontop() 
{
	if(	m_StayOnTop )
		m_StayOnTop = FALSE;
	else
		m_StayOnTop = TRUE;
	StayOnTop();
}
void CMainFrame::OnUpdateViewStayontop(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_StayOnTop);
	
}
void CMainFrame::StayOnTop()
{
CWnd* pWnd = AfxGetMainWnd();
	ASSERT_VALID(pWnd);
	pWnd->SetWindowPos( (m_StayOnTop) ? &wndTopMost : &wndNoTopMost,
						0,0,0,0, SWP_NOMOVE|SWP_NOSIZE);
}

////////////////////////////////////////////////////////////////////////
//  Called to service the 1 Second status timer updates
////////////////////////////////////////////////////////////////////////
void CMainFrame::OnTimer(UINT nIDEvent) 
{
	if( nIDEvent == m_TimerID )
		UpdateStatus();
	CFrameWnd::OnTimer(nIDEvent);
}

///////////////////////////////////////////////////////////////////////
//  Update the status bar info every timer period
///////////////////////////////////////////////////////////////////////
void CMainFrame::UpdateStatus()
{
CAO40RcvDoc* pDoc = (CAO40RcvDoc*)CMainFrame::GetActiveDocument();
	ASSERT_VALID(pDoc);
CString sBuf;
UINT nID, nStyle;
INT nWidth, nIndex;
CSize szExtent;
	CClientDC dc(&m_wndStatusBar);
	CFont* pOldFont = dc.SelectObject(m_wndStatusBar.GetFont());

// Get current time and date
CTime time = CTime::GetCurrentTime();
	sBuf = time.FormatGmt(_T(" %#d %b %Y     %#H:%M:%S UTC "));
	nIndex = m_wndStatusBar.CommandToIndex(IDS_STATUSBARX2);
	szExtent = dc.GetTextExtent(sBuf, sBuf.GetLength() );
	m_wndStatusBar.GetPaneInfo(nIndex, nID, nStyle, nWidth);
	m_wndStatusBar.SetPaneInfo(nIndex, nID, nStyle, szExtent.cx);
	m_wndStatusBar.SetPaneText(nIndex, sBuf);

// Figure out Status text string to display
	switch(pDoc->m_RecvState)
	{
		case STATE_NOSIG:
			sBuf = _T(" Looking for Signal ");
			break;
		case STATE_SYNC:
			sBuf = _T(" Searching for Sync ");
			break;
		case STATE_READ:
			sBuf = _T("    Reading Block   ");
			break;
		case STATE_TOOSLOW:
			sBuf = _T(" CPU too slow ");
			break;
		case STATE_STOPPED:
			sBuf =_T("Stopped");
			break;
		default:
			sBuf= _T("");
			break;
	}

	szExtent = dc.GetTextExtent(sBuf, sBuf.GetLength() );
	nIndex = m_wndStatusBar.CommandToIndex(IDS_STATUSBARX1);
	m_wndStatusBar.GetPaneInfo(nIndex, nID, nStyle, nWidth);
	m_wndStatusBar.SetPaneInfo(nIndex, nID, nStyle, szExtent.cx);
	m_wndStatusBar.SetPaneText(nIndex, sBuf);	
	if( pDoc->m_NetOn)
	{
		CAO40RcvView* pView = (CAO40RcvView*)GetActiveView();
		ASSERT_VALID(pView);
		pView->UpdateNetStatus(pDoc);
	}
	dc.SelectObject(pOldFont);	//restore font
}

////////////////////////////////////////////////////////////////////////
//  Called to invode the window's soundcard mixer program
////////////////////////////////////////////////////////////////////////
void CMainFrame::OnInputLevelAdj() 
{
	::WinExec(_T("SndVol32.exe /r"), SW_SHOW);	
}

////////////////////////////////////////////////////////////////////////
//  Called to invoke the Output Settings Dialog
////////////////////////////////////////////////////////////////////////
void CMainFrame::OnOutputSettings() 
{
CAO40RcvDoc* pDoc = (CAO40RcvDoc*)CMainFrame::GetActiveDocument();
	ASSERT_VALID(pDoc);
CAO40RcvView* pView = (CAO40RcvView*)GetActiveView();
	ASSERT_VALID(pView);
CDispDlg dlg;
CString str;
	dlg.m_crRcvBackColor = pDoc->m_crRcvBackColor;
	dlg.m_crRcvTextColor = pDoc->m_crRcvTextColor;
	dlg.m_lfRcvFont = pDoc->m_lfRcvFont;
	dlg.m_RawLogOn = pDoc->m_RawLogOn;
	dlg.m_TlmLogOn = pDoc->m_TlmLogOn;
	dlg.m_NetOn = pDoc->m_NetOn;
	dlg.m_NetSocket = pDoc->m_NetSocket;
	dlg.m_QualifyCRC = pDoc->m_QualifyCRC;
	dlg.m_UseABlocks = pDoc->m_UseABlocks;
	dlg.m_UseEBlocks = pDoc->m_UseEBlocks;
	dlg.m_SockDataFormat = pDoc->m_SockDataFormat;
	dlg.m_CallSign = pDoc->m_CallSign;
	dlg.m_OutWavePath = pDoc->m_OutWavePath;
	dlg.m_UseWaveOut = pDoc->m_UseWaveOut;
	dlg.m_UseTimeStamp = pDoc->m_UseTimeStamp;
	dlg.m_TelemPath = pDoc->m_TelemPath;
	dlg.m_AutoViewToggle = pDoc->m_AutoViewToggle;	
	dlg.m_WaveAppend = pDoc->m_WaveAppend;	
	if (dlg.DoModal() == IDOK)
	{
		pDoc->m_crRcvBackColor = dlg.m_crRcvBackColor;
		pDoc->m_crRcvTextColor = dlg.m_crRcvTextColor;
		pDoc->m_lfRcvFont = dlg.m_lfRcvFont;
		pDoc->m_RawLogOn = dlg.m_RawLogOn;
		pDoc->m_TlmLogOn = dlg.m_TlmLogOn;
		pDoc->m_NetSocket = dlg.m_NetSocket;
		pDoc->m_QualifyCRC = dlg.m_QualifyCRC;
		pDoc->m_UseABlocks = dlg.m_UseABlocks;
		pDoc->m_UseEBlocks = dlg.m_UseEBlocks;
		pDoc->m_SockDataFormat = dlg.m_SockDataFormat;
		pDoc->m_CallSign = dlg.m_CallSign;
		pDoc->m_OutWavePath = dlg.m_OutWavePath;
		pDoc->m_UseWaveOut = dlg.m_UseWaveOut;
		pDoc->m_UseTimeStamp = dlg.m_UseTimeStamp;
		pDoc->m_TelemPath = dlg.m_TelemPath;
		pDoc->m_AutoViewToggle = dlg.m_AutoViewToggle;
		pDoc->m_WaveAppend = dlg.m_WaveAppend;
		if((pDoc->m_NetOn != dlg.m_NetOn) && pDoc->m_NetworkAvailable && dlg.m_NetOn)
		{
			pDoc->StartSocket();
		}
		pDoc->m_NetOn = dlg.m_NetOn;
		pView->m_RxTextCtrl.SetCharFormat( pDoc->m_lfRcvFont,
										pDoc->m_crRcvTextColor,
										pDoc->m_crRcvBackColor	);
		pView->m_RxTextCtrl.SetText( pDoc->GetOutputText() );
		if(pDoc->m_InGroup == INPUT_SOUND && pDoc->m_UseWaveOut)
			pView->StopProcess();
		pView->SetButtonTypes();

	}	
	str = _T("AO40Rcv - Ver. 2.04 - ") + pDoc->m_CallSign;
	AfxGetMainWnd()->SetWindowText( str );
}

////////////////////////////////////////////////////////////////////////
//  Called to invoke the Rig Control Dialog
////////////////////////////////////////////////////////////////////////
void CMainFrame::OnRigCtrl() 
{
CAO40RcvDoc* pDoc = (CAO40RcvDoc*)CMainFrame::GetActiveDocument();
	ASSERT_VALID(pDoc);
CAO40RcvView* pView = (CAO40RcvView*)GetActiveView();
	ASSERT_VALID(pView);
CRigDlg dlg;
	dlg.m_ComPort = pDoc->m_ComPort;
	dlg.m_RigModeGroup = pDoc->m_RigModeGroup & ~(RIGCTRL_MODE_ACTIVEMASK);
	dlg.m_PortParams = pDoc->m_PortParams;
	dlg.m_USBMode = pDoc->m_USBMode;
	dlg.m_CenterFrequency = pDoc->m_CenterFrequency;
	dlg.m_UPString = pDoc->m_UPString;
	dlg.m_DNString = pDoc->m_DNString;
	dlg.m_StepSize = pDoc->m_StepSize;
	fnSetRigCtrlOptions(0,0,0,0,0,100);	//stop rig control process
	if (dlg.DoModal() == IDOK)
	{
		pDoc->m_ComPort = dlg.m_ComPort;
		if(pDoc->m_ComPort != 0 )
			pDoc->m_RigModeGroup = dlg.m_RigModeGroup | RIGCTRL_MODE_ACTIVEMASK;
		else
			pDoc->m_RigModeGroup = dlg.m_RigModeGroup;
		pDoc->m_PortParams = dlg.m_PortParams;
		pDoc->m_USBMode = dlg.m_USBMode;
		pDoc->m_UPString = dlg.m_UPString;
		pDoc->m_DNString = dlg.m_DNString;
		pDoc->m_CenterFrequency = dlg.m_CenterFrequency;
		pDoc->m_StepSize = dlg.m_StepSize;
		fnSetCmdStrings(pDoc->m_UPString,pDoc->m_DNString);
	}
	pView->SetDopplerMode(pDoc);
}

////////////////////////////////////////////////////////////////////////
//  Called to invoke the Input Select Dialog
////////////////////////////////////////////////////////////////////////
void CMainFrame::OnInputSelect() 
{
CAO40RcvDoc* pDoc = (CAO40RcvDoc*)CMainFrame::GetActiveDocument();
	ASSERT_VALID(pDoc);
CAO40RcvView* pView = (CAO40RcvView*)GetActiveView();
	ASSERT_VALID(pView);
CInSelect dlg;
	dlg.m_InGroup = pDoc->m_InGroup;
	dlg.m_InWaveName = pDoc->m_InWaveName;
	dlg.m_InWavePath = pDoc->m_InWavePath;
	dlg.m_InRawName = pDoc->m_InRawName;
	dlg.m_InRawPath = pDoc->m_InRawPath;
	dlg.m_SoundCard = pDoc->m_SoundCard;
	dlg.m_NumBlocksInFile = pDoc->m_NumBlocksInFile;
	dlg.m_WaveStartOffset = pDoc->m_WaveStartOffset;
	if (dlg.DoModal() == IDOK)
	{
		pDoc->m_InGroup = dlg.m_InGroup;
		pDoc->m_InWaveName = dlg.m_InWaveName;
		pDoc->m_InWavePath = dlg.m_InWavePath;
		pDoc->m_InRawName = dlg.m_InRawName;
		pDoc->m_InRawPath = dlg.m_InRawPath;
		pDoc->m_SoundCard = dlg.m_SoundCard;
		pDoc->m_NumBlocksInFile = dlg.m_NumBlocksInFile;
		pDoc->m_WaveStartOffset = dlg.m_WaveStartOffset;
		if(pDoc->m_InGroup != INPUT_SOUND)
			pView->StopProcess();
		pView->InitTlmRead();
		pView->SetButtonTypes();
		pView->SetDopplerMode(pDoc);
	}		
}

BOOL CMainFrame::PreTranslateMessage(MSG* pMsg) 
{
CAO40RcvView* pView = (CAO40RcvView*)GetActiveView();
	ASSERT_VALID(pView);
	switch (pMsg->message)
	{
		case WM_KEYDOWN:
		case WM_SYSKEYDOWN:
			switch (pMsg->wParam)
			{
				case VK_DOWN:
					pView->SetDopplerCorrection(0);
					break;
				case VK_LEFT:
					pView->ExtLeftKey();
					break;
				case VK_UP:
					pView->SetDopplerCorrection(0);
					break;
				case VK_RIGHT:
					pView->ExtRightKey();
					break;
				case VK_NEXT:
				case VK_PRIOR:
					break;
				case VK_F1:
					pView->SetTABExternal(TAB_SIG_VIEW0);
					break;
				case VK_F2:
					pView->SetTABExternal(TAB_STAT_VIEW1);
					break;
				case VK_F3:
					pView->SetTABExternal(TAB_MATRIX_VIEW2);
					break;
				case VK_F4:
					pView->SetTABExternal(TAB_NAV_VIEW3);
					break;
				case VK_F5:
					pView->SetTABExternal(TAB_PWR_VIEW4);
					break;
				case VK_F6:
					pView->SetTABExternal(TAB_TEMP_VIEW5);
					break;
				case VK_F7:
					pView->SetTABExternal(TAB_MAIL_VIEW6);
					break;
				case VK_F8:
					pView->SetTABExternal(TAB_RAW_VIEW7);
					break;
				case VK_F9:
				case VK_F10:
				case VK_F11:
					break;
				case VK_F12:
					pView->ExtStartStop();
					break;
			}
			break;
	}		
	return CFrameWnd::PreTranslateMessage(pMsg);
}
