// TLMdecode.cpp: implementation of the CTLMdecode class.
//
//////////////////////////////////////////////////////////////////////
// Copyright 2000.    Moe Wheatley AE4JY  <ae4jy@qsl.net>
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "AO40Rcv.h"
#include "TLMdecode.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#define DATABLOCK_SIZE 514

#define PI 3.141592654

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CTLMdecode::CTLMdecode()
{

}

CTLMdecode::~CTLMdecode()
{

}


/////////////////////////////////////////////////////////////////////////////
//  Routines for displaying all the interpreted data
/////////////////////////////////////////////////////////////////////////////
void CTLMdecode::MakeRawStr(CString& str, BOOL crcok)
{
CString s;
	str = "adr-  0   1   2   3   4   5   6   7     8   9   A   B   C   D   E   F";
	if( crcok)
	{
		for(INT i=0;i<512;i++)
		{
			if( (i%16) == 0 )
			{
				s.Format("\n%3.3X-[%2.2X]", i, m_pDat[i]);
			}
			else
			{
				if( (i&0x0F) == 8 )
					s.Format("  [%2.2X]",m_pDat[i]);
				else
					s.Format("[%2.2X]",m_pDat[i]);
			}
			str += s;
		}
	}
	else
	{
		for(INT i=0;i<512;i++)
		{
			if( (i%16) == 0 )
			{
				s.Format("\n%3.3X-?%2.2X]", i, m_pDat[i]);
			}
			else
			{
				if( (i&0x0F) == 8 )
					s.Format("  ?%2.2X]",m_pDat[i]);
				else
					s.Format("?%2.2X]",m_pDat[i]);
			}
			str += s;
		}

	}
}

void CTLMdecode::MakeMailStr(CString& str, BOOL crcok)
{
INT j=0;
INT n;
	str = "";
	for(INT i=1;i<=512;i++)
	{
		n = str.Insert(j++, m_pDat[i-1]);
		ASSERT(n == str.GetLength());
   		if( (i%64) == 0)
		{
			n = str.Insert(j++, '\n' );
			ASSERT(n == str.GetLength());
		}
	}
	if(!crcok)
	{
		n = str.Insert(0, "Bad CRC ");
		ASSERT(n == str.GetLength());
	}
}

void CTLMdecode::MakeStatusAStr( CString& str, BOOL crcok)
{
INT j=66;
INT n;
	if(crcok)
		str = "+=+=+=+=+=+=+=+=+=+=  M A I N    S T A T U S  +=+=+=+=+=+=+=+=+=\n\n";
	else
		str = "+=+=+Bad CRC+=+=+=+=  M A I N    S T A T U S  +=+=+=+Bad CRC+=+=\n\n";
	for(INT i=1;i<=256;i++)
	{
		n = str.Insert(j++, m_pDat[i-1]);
		ASSERT(n == str.GetLength());
   		if( (i%64) == 0)
		{
			n = str.Insert(j++, '\n' );
			ASSERT(n == str.GetLength());
		}
	}
	MakeStatusScrn(str);
}

void CTLMdecode::MakeStatusEStr( CString& str, BOOL crcok)
{
INT j=66;
INT n;
	if(crcok)
		str = "+=+=+=+=+=+=+=+=+=+  E V E N T    S T A T U S  +=+=+=+=+=+=+=+=+\n";
	else
		str = "+=+=+Bad CRC+=+=+=+  E V E N T    S T A T U S  +=+=+Bad CRC+=+=+\n";
	for(INT i=1;i<=256;i++)
	{
		n = str.Insert(j++, m_pDat[i-1]);
		ASSERT(n == str.GetLength());
   		if( (i%64) == 0)
		{
			n = str.Insert(j++, '\n' );
			ASSERT(n == str.GetLength());
		}
	}
	MakeStatusScrn(str);
}

void CTLMdecode::MakeStatusScrn(CString &str)
{
CString s;
Uword y;
CTime t0( 1978, 1, 1, 0, 0, 0 ); // 00:00:00 Jan 1, 1978 AMSAT day 0
CTime tl = CTime::GetCurrentTime();
struct tm* osTime; 
osTime = tl.GetGmtTm( NULL );
CTime t(osTime->tm_year+1900,osTime->tm_mon+1,osTime->tm_mday,osTime->tm_hour,
				osTime->tm_min,osTime->tm_sec);
CTimeSpan tc = t - t0;  // Subtract 2 CTimes
	GetAmsatDay( t );
CTimeSpan ts = t - t0;  // Subtract 2 CTimes
	y.bytes.lsb = m_pDat[0x1A6]; y.bytes.msb = m_pDat[0x1A7];
	s.Format("\nAMSAT Day = %-5d", ts.GetDays()); str += s;
	s.Format("   Orbit# = %-5d  MA = %3d",y.both,m_pDat[0x1A5]);str += s;
	s.Format("\nCurrent AMSAT Day = %d", tc.GetDays()); str += s;
	s = "\n--------------------  R E C E I V E R S -------------------------\n"; str += s;
	s.Format("21MHz:%s 24MHz:%s V:%s    U:%s\n",GetMsg(0x1CA,BIT0,MSG_ONOFF),
				GetMsg(0x1CA,BIT1,MSG_ONOFF),GetMsg2(0x1CA,BIT2,0x189,BIT0,MSG_RXANT1),
				GetMsg2(0x1CA,BIT2,0x189,BIT1,MSG_RXANT2));str += s;
	s.Format("L1:%s    L2:%s    S1:%s\n",GetMsg2(0x189,BIT2,0x189,BIT3,MSG_LANT),
				GetMsg(0x1CA,BIT5,MSG_ONOFF), GetMsg(0x1CA,BIT6,MSG_ONOFF) );str += s;
	s.Format("S2:%s    C:%s     Leila: #%2.2X%2.2X",GetMsg(0x1CA,BIT3,MSG_ONOFF),GetMsg(0x1CA,BIT7,MSG_ONOFF),
				m_pDat[0x1D1],m_pDat[0x1D0] ); str += s;
	s = "\n------------------  T R A N S M I T E R S -----------------------\n"; str += s;
	if(m_pDat[0x189] & BIT1) //higain
	{
		s.Format("U:%s",GetMsg2(0x1C8,BIT2,0x1C9,BIT2,MSG_UHTXANT));str += s;
	}
	else		//lo gain
	{
		s.Format("U:%s",GetMsg2(0x1C8,BIT2,0x1C9,BIT2,MSG_UOTXANT));str += s;
	}
	s.Format(" V:%s S1:%s S2:%s\n",GetMsg2(0x1C9,BIT1,0x189,BIT0,MSG_VTXANT),
			GetMsg(0x1C9,BIT3,MSG_ONOFF),GetMsg(0x1C8,BIT0,MSG_ONOFF) );str += s;
	s.Format("X:%s K:%s",GetMsg2(0x1C8,BIT5,0x1C9,BIT5,MSG_XTXANT),
			GetMsg2(0x1C9,BIT6,0x1C8,BIT6,MSG_KTXANT) );str += s;
	s = "\n----------------------  A G C / A L C --------------------------\n"; str += s;
	s.Format("V-Rx: %-5.2fdB       U-Rx: %-5.2fdB      L1-Rx: %-5.2fdB     \n",
				AGC1( m_pDat[0x11E]),AGC2( m_pDat[0x11F]),AGC3( m_pDat[0x120]) );str += s;
	s.Format("L2-Rx: %-5.2fdB       S1-Rx: %-5.2fdB    S2/C-Rx: %-5.2fdB     \n",
				AGC4( m_pDat[0x116]),AGC5( m_pDat[0x11D]),AGC6( m_pDat[0x11B]) );str += s;
	s.Format("HF-Rx: %3.3d      V-Tx: %3.3d     U-Tx: %3.3d     S2-Tx: %3.3d", 
			m_pDat[0x11C],m_pDat[0x124],m_pDat[0x126],m_pDat[0x139] );str += s;
	s = "\n----------------------  B E A C O N S ---------------------------\n"; str += s;
	s.Format("GB:%sGB FSK:%sDPSK:%sEB:%s\n",GetMsg(0x194,BIT0,MSG_OFFON),GetMsg(0x194,BIT1,MSG_ONOFF),
		GetMsg(0x194,BIT2,MSG_OFFON),GetMsg(0x194,BIT3,MSG_ONOFF)  );str += s;
		s.Format("GB PSK Source:%s",GetMsg2(0x194,BIT4,0x194,BIT5,MSG_BEAC1));str += s;
	s = "\n-----------------------  S A F E T Y ---------------------------\n"; str += s;
	s.Format("Magnet System:%s Emerg. Flags:#%2.2X\n",GetMsg2(0x1A0,BIT1,0x1A2,BIT3,MSG_MAGSYS),
					m_pDat[0x1ED]); str += s;
	s.Format("Soft Errors:%d            Solar Array Plug:%s",(m_pDat[0x1D9]>>5)&0x7,
					GetMsg(0x1D9,BIT1,MSG_SARRAY)); str += s;
	s = "\n------------------  E X P E R I M E N T S -----------------------\n"; str += s;
	s.Format("Rudak:%s   GPS:%s  RFexp:%sCamera A:%s\n",GetMsg(0x188,BIT1,MSG_ONOFF),
			GetMsg(0x188,BIT2,MSG_ONOFF),GetMsg(0x188,BIT3,MSG_ONOFF),
			GetMsg(0x188,BIT4,MSG_ONOFF) );str += s;
	s.Format("Camera B:%sCedex:%sCedexCtrl:%sIHU-2:%s",GetMsg(0x188,BIT5,MSG_ONOFF),
			GetMsg(0x188,BIT7,MSG_ONOFF),GetMsg(0x188,BIT6,MSG_ONOFF),
			GetMsg(0x1C9,BIT0,MSG_ONOFF) );str += s;
}

void CTLMdecode::MakeNavStr(CString &str, BOOL crcok)
{
CString s="";
	if(crcok)
		str = "+=+=+=+=+=+=+=+=+=+=  N A V I G A T I O N   +=+=+=+=+=+=+=+=+=+=\n\n";
	else
		str = "+=+=+Bad CRC+=+=+=+=  N A V I G A T I O N   +=+=+=+Bad CRC+=+=+=\n\n";
	s =   "--------------------- S T A B I L I T Y --------------------------\n"; str += s;
	s.Format("SEU Sensor Mode:%s",GetMsg(0x19F,BIT0,MSG_MODE1)); str += s;
	s.Format("Magnet Sys:%s\n",GetMsg(0x1A0,BIT0,MSG_ONOFF)); str += s;
	s.Format("Magnet Pwr:%s",GetMsg(0x1A2,BIT3,MSG_ONOFF)); str += s;
	s.Format("M-SOLL: %3.3d\n",m_pDat[0x1A1]); str += s;
	s =   "----------------- E A R T H   S E N S O R S ----------------------\n"; str += s;
	s.Format("Sensitivity:%-7.2fmV    ",CalcSEUSens()); str +=s;
	s.Format("Pointing:%s\n",GetPointing() ); str +=s;
	s.Format("Edge:%s   Lock Range: %3.3d\n",GetMsg(0x1C6,BIT2,MSG_PN),m_pDat[0x19A] ); str += s;
	s.Format("ES-R Upper--> MA:%3.3d Orbit:%3.3d(mod256)\n",m_pDat[0x196],m_pDat[0x197]); str +=s;
	s.Format("ES-R Lower--> MA:%3.3d Orbit:%3.3d(mod256)\n",m_pDat[0x198],m_pDat[0x199]); str +=s;
	s.Format("ES1 spin Cnt: %3.3d   Update Flag1 %3.3d\n",m_pDat[0x19B],m_pDat[0x19C]); str += s;
	s.Format("ES2 spin Cnt: %3.3d   Update Flag2 %3.3d\n",m_pDat[0x19D],m_pDat[0x19E]); str += s;
	s =   "-------------- S P I N   S U N    S E N S O R S ------------------\n"; str += s;
	s.Format("SS1 cnt:%3.3d     SS2 cnt:%3.3d     Threshold: %-5.2fV\n",m_pDat[0x190],
						m_pDat[0x191],CalcSunSens() ); str +=s;
	s.Format("SS-Flags: %3.3d   Spin Count Raw: %3.3d  SEU: %-7.2fRPM\n",m_pDat[0x192],
					m_pDat[0x193], BR1( m_pDat[0x100]) ); str += s;
	s.Format("Sun Angle: %-6.2f deg.\n",CalcSunAngle() ); str += s;
	s =   "------------- 3 - A X I S   S U N   S E N S O R S ----------------\n"; str += s;
	if( m_pDat[0x136]>245 )
	{
		s.Format("Angle Y-25: %-5.2f deg.     Z-25: %-5.2f deg.\n",SSA1( m_pDat[0x12F])
						,SSA1( m_pDat[0x12E])); str += s;
	}
	if( m_pDat[0x137]>245 )
	{
		s.Format("Angle Y-45: %-5.2f deg.     Z-45: %-5.2f deg.\n",SSA2( m_pDat[0x135])
						,SSA2( m_pDat[0x134])); str += s;
	}
	s.Format("Omni Up-X: %-5.2f deg.      Up-Y: %-5.2f deg.\n",SSA3( m_pDat[0x130])
						,SSA3( m_pDat[0x131])); str += s;
	s.Format("Omni Down-X: %-5.2f deg.    Down-Y: %-5.2f deg.\n",SSA3( m_pDat[0x132])
						,SSA3( m_pDat[0x133])); str += s;
	s.Format("U//D Z: %s Plane\n", GetMsg(0x12D,BIT7,MSG_UPDN) ); str += s;

	if( m_pDat[0x1D9]&BIT2 )
	{
		s =   "------ E L E C T R O N I C   P R O P U L S I O N   U N I T--------\n"; str += s;
		s.Format("                    Configuration: #%2.2X\n",m_pDat[0x181] ); str += s;
		s.Format("Tank Pressure:%7.2f Bar.        Motor Pressure:%7.2f Bar.\n",
						BAR1(m_pDat[0x101]), BAR2(m_pDat[0x102]) ); str += s;
		s.Format("Motor Voltage:%7.2f V       Motor Current:%7.2f A\n",
						BV5(m_pDat[0x104]), BI10(m_pDat[0x103]) ); str += s;
		s.Format("Set Current:%7.2f A   Flow(analog): %d   Flow Rate: %d\n",
					BI11(m_pDat[0x185]), m_pDat[0x105], m_pDat[0x186] ); str += s;
	}
	if( m_pDat[0x1C6]&BIT3 )
	{
		s =   "------------ L I Q U I D   I G N I T I O N   U N I T -------------\n"; str += s;
		s.Format("High Pressure:%7.2f Bar.        Low Pressure:%7.2f Bar.\n",
						BAR3(m_pDat[0x114]), BAR4(m_pDat[0x115]) ); str += s;
		if(m_pDat[0x113]>89)
			str += "Motor Valve Position: Open\n";
		else
			str += "Motor Valve Position: Closed\n";
	}
	s =   "--------------- R E A C T I O N    W H E E L S -------------------\n"; str += s;
	s.Format("Wheel 1 Power:%s  Speed: %7.2f RPM",	GetMsg(0x187,BIT0,MSG_ONOFF),
					CalcWRPM(0x1C0) ); str += s;
	if( m_pDat[0x187]&BIT0 )
	{
		s.Format("    Sync:%s",GetMsg(0x1DA,BIT2,MSG_YN)  ); str += s;
	}
	s.Format("\nWheel 2 Power:%s  Speed: %7.2f RPM",	GetMsg(0x187,BIT1,MSG_ONOFF),
					CalcWRPM(0x1C2) ); str += s;
	if( m_pDat[0x187]&BIT1 )
	{
		s.Format("    Sync:%s",GetMsg(0x1DA,BIT3,MSG_YN)  ); str += s;
	}
	s.Format("\nWheel 3 Power:%s  Speed: %7.2f RPM",	GetMsg(0x187,BIT2,MSG_ONOFF),
					CalcWRPM(0x1C4) ); str += s;
	if( m_pDat[0x187]&BIT2 )
	{
		s.Format("    Sync:%s",GetMsg(0x1DA,BIT4,MSG_YN)  ); str += s;
	}
}

void CTLMdecode::MakeTempStr(CString &str, BOOL crcok)
{
CString s;
	if(crcok)
		str = "+=+=+=+=+=+=+=+=+=  T E M P E R A T U R E S   +=+=+=+=+=+=+=+=+=\n\n";
	else
		str = "+=+=+Bad CRC+=+=+=  T E M P E R A T U R E S   +=+=+Bad CRC+=+=+=\n\n";
	s =   "--------------------- H E A T   P I P E S ------------------------\n"; str += s;
	s.Format("HP1+X-Y: %-7.2f  HP2+X+Y: %-7.2f  HP2+X-Y: %-7.2f  HP2-X: %-7.2f\n",
				BT( m_pDat[0x169]), BT( m_pDat[0x168]), BT( m_pDat[0x153] ),BT( m_pDat[0x154])); str += s;
	s.Format("HP3+X: %-7.2f    HP3-X: %-7.2f    HP4+X+Y: %-7.2f\n",
				BT( m_pDat[0x16A]), BT( m_pDat[0x167]), BT( m_pDat[0x166] )); str += s;
	s = "\n---------------------- S / C   F R A M E -------------------------\n"; str += s;
	s.Format("Top: %-7.2f      Bottom: %-7.2f   Back: %-7.2f     Side2: %-7.2f\n",
				BT( m_pDat[0x162]), BT( m_pDat[0x163]), BT( m_pDat[0x164] ),BT( m_pDat[0x16D])); str += s;
	s.Format("Side4: %-7.2f    S Ant: %-7.2f\n",
				BT( m_pDat[0x165]), BT( m_pDat[0x16E])); str += s;
	s = "\n------------------------- T A N K S ------------------------------\n"; str += s;
	s.Format("MMH B3: %-7.2f   N2O4+X+Y: %-7.2f N2O4-X-Y: %-7.2f\n",
				BT( m_pDat[0x145]), BT( m_pDat[0x16C]), BT( m_pDat[0x16B] )); str += s;
	s.Format("NH3 B2: %-7.2f   Helium: %-7.2f\n",
				BT( m_pDat[0x148]), BT( m_pDat[0x16F])); str += s;
	s = "\n--------------------- B A T T E R I E S --------------------------\n"; str += s;
	s.Format("Main B2: %-7.2f  Main B4: %-7.2f  Main B6: %-7.2f\n",
				BT( m_pDat[0x149]), BT( m_pDat[0x14A]), BT( m_pDat[0x14B] )); str += s;
	s.Format("Aux B1: %-7.2f   Aux B5: %-7.2f\n",
				BT( m_pDat[0x147]), BT( m_pDat[0x146])); str += s;
	s = "\n------------------ S O L A R   P A N E L S -----------------------\n"; str += s;
	s.Format("Panel 1: %-7.2f  Panel 2: %-7.2f  Panel 3: %-7.2f\n",
				BT( m_pDat[0x14C]), BT( m_pDat[0x14D]), BT( m_pDat[0x14E] )); str += s;
	s.Format("Panel 4: %-7.2f  Panel 5: %-7.2f  Panel 6: %-7.2f\n",
				BT( m_pDat[0x14F]), BT( m_pDat[0x150]), BT( m_pDat[0x151] )); str += s;
	s = "\n------------------ B A T T E R Y   R E G S -----------------------\n"; str += s;
	s.Format("BCR 1: %-7.2f    BCR 2: %-7.2f    BCR 3: %-7.2f\n",
				BT( m_pDat[0x142]), BT( m_pDat[0x144]), BT( m_pDat[0x143] )); str += s;
	s = "\n---------------------- M O D U L E S -----------------------------\n"; str += s;
	s.Format("IHU: %-7.2f      SEU: %-7.2f      EPU: %-7.2f\n",
				BT( m_pDat[0x161]), BT( m_pDat[0x140]), BT( m_pDat[0x141] )); str += s;
	s = "\n------------------ T R A N S M I T T E R S -----------------------\n"; str += s;
	s.Format("V Tx: %-7.2f     U Tx: %-7.2f     U PA: %-7.2f     S1 Tx: %-7.2f\n",
				BT( m_pDat[0x15E]), BT( m_pDat[0x158]), BT( m_pDat[0x15F] ),BT( m_pDat[0x15B])); str += s;
	s.Format("S2 Tx: %-7.2f    X Tx: %-7.2f     TWTA X Tx: %-7.2f\n",
				BT( m_pDat[0x15C]), BT2( m_pDat[0x12B]), BT2( m_pDat[0x12C] )); str += s;
	s = "\n--------------------- R E C E I V E R S --------------------------\n"; str += s;
	s.Format("U&V: %-7.2f      L1: %-7.2f       L2: %-7.2f\n",
				BT( m_pDat[0x159]), BT( m_pDat[0x15A]), BT( m_pDat[0x152] )); str += s;
	s.Format("S1-HF: %-7.2f    S2-C: %-7.2f\n",
				BT( m_pDat[0x157]), BT( m_pDat[0x156])); str += s;
}
void CTLMdecode::MakePwrStr(CString &str, BOOL crcok)
{
CString s;
	if(crcok)
		str = "+=+=+=+=+=+=+=+=+=+=+=+=  P O W E R   +=+=+=+=+=+=+=+=+=+=+=+=+=\n\n";
	else
		str = "+=+=+Bad CRC+=+=+=+=+=+=  P O W E R   +=+=+=+=+=+=+Bad CRC+=+=+=\n\n";
	s =   "----------------------- S T A T U S ------------------------------\n"; str += s;
	s.Format("Batteries:%sCharger:%sAux Heater:%s\n",GetMsg2(0x18B,BIT0,0x18B,BIT1,MSG_BATSTAT),
		GetMsg(0x18B,BIT2,MSG_ONOFF),GetMsg(0x18B,BIT3,MSG_ONOFF)); str += s;
	s.Format("Solar Array Release Unit:%s",GetMsg(0x188,BIT0,MSG_ONOFF));str += s;

	s = "\n---------------------- V O L T A G E S ---------------------------\n"; str += s;
	s.Format("Main Bat:%7.2fV   Aux Bat:%7.2fV   28V Bus:%7.2fV\n",
				BV1( m_pDat[0x10B]), BV1( m_pDat[0x10C]), BV1( m_pDat[0x10D] )); str += s;
	s.Format("Bat Voltage Offset:%7.2fV\n",CalcBatOffset() ); str += s;
	s = "\n---------------------- C U R R E N T S ---------------------------\n"; str += s;
	s.Format("Total Bat:%7.2fA  EPU:%7.2fA       Main Bus:%7.2fA\n",
				BI3(m_pDat[0x106]), BI4( m_pDat[0x107]), BI5( m_pDat[0x108] )); str += s;
	s.Format("28v-S:%7.2fA      BCR:%7.2fA       SEU:[%3.3d]\n",
				BI6(m_pDat[0x109]), BI7( m_pDat[0x10A]), m_pDat[0x170] ); str += s;
	s = "\n------ B A T T E R Y   C H A R G E   R E G U L A T O R  # 1 ------\n"; str += s;
	s.Format("Vin:%7.2fV        I SA-1:%7.2fA    I SA-6:%7.2fA\n",
				BV2( m_pDat[0x10E]), BI1( m_pDat[0x171]), BI1( m_pDat[0x172] )); str += s;
	s.Format("BCR-1 Array Voltage Offset:%7.2fV\n",CalcBCROffset(0x18D) ); str += s;
	s = "\n------ B A T T E R Y   C H A R G E   R E G U L A T O R  # 2 ------\n"; str += s;
	s.Format("Vin:%7.2fV        10V-C1:%7.2fV\n",
				BV3( m_pDat[0x111]), BV4( m_pDat[0x112])); str += s;
	s.Format("I SA-4:%7.2fA     I SA-5:%7.2fA    I 10v-C1:%7.2fA\n",
				BI1( m_pDat[0x176]), BI1( m_pDat[0x177]), BI2( m_pDat[0x178] )); str += s;
	s.Format("BCR-2 Array Voltage Offset:%7.2fV\n",CalcBCROffset(0x18E) ); str += s;
	s = "\n------ B A T T E R Y   C H A R G E   R E G U L A T O R  # 3 ------\n"; str += s;
	s.Format("Vin:%7.2fV        10V-C2:%7.2fV\n",
				BV3( m_pDat[0x10F]), BV4( m_pDat[0x110])); str += s;
	s.Format("I SA-2:%7.2fA     I SA-3:%7.2fA    I 10v-C2:%7.2fA\n",
				BI1( m_pDat[0x174]), BI1( m_pDat[0x173]), BI2( m_pDat[0x175] )); str += s;
	s.Format("BCR-3 Array Voltage Offset:%7.2fV\n",CalcBCROffset(0x18F) ); str += s;
	s = "\n---------------------- T X   P O W E R ---------------------------\n"; str += s;
	s.Format("V-Tx:[%3.3d]          U-Amp:[%3.3d]        I-28v-S:%7.2fA\n",
				m_pDat[0x123], m_pDat[0x125], BI8( m_pDat[0x17A] )); str += s;
	s.Format("I-10v-S:[%3.3d]       S1-A/mix:[%3.3d]     S2 Pwr:[%3.3d]\n",
				m_pDat[0x17B], m_pDat[0x122], m_pDat[0x138] ); str += s;
	s.Format("CS X-Tx:[#%2.2X]      Out X-Tx:[%3.3d]     Helix X Tx:%7.2fmA\n",
				m_pDat[0x118], m_pDat[0x117], BI9( m_pDat[0x119] )); str += s;
	s.Format("K-Tx:[%3.3d]          I-K-Tx:[%3.3d]\n",
				m_pDat[0x11A], m_pDat[0x179] ); str += s;

}
void CTLMdecode::MakeMatrixStr(CString &str, BOOL crcok)
{
CString s;
	if(crcok)
		str = "=+=+=+=+=+=+= I F   M A T R I X  +=+=+=+=+=+=+=+\n\n";
	else
		str = "=+Bad CRC+=+= I F   M A T R I X  +=+=+Bad CRC+=+\n\n";
	CalcIFMatrix();		//creates ascii matrix array
	s = "Inputs\n";str += s;
	s = "24/21"; s += (LPCTSTR)m_IFMatrixStr[0]; str+=s;
	s = "    V"; s += (LPCTSTR)m_IFMatrixStr[1]; str+=s;
	s = "    U"; s += (LPCTSTR)m_IFMatrixStr[2]; str+=s;
	s = "   L1"; s += (LPCTSTR)m_IFMatrixStr[3]; str+=s;
	s = "   L2"; s += (LPCTSTR)m_IFMatrixStr[4]; str+=s;
	s = "   S1"; s += (LPCTSTR)m_IFMatrixStr[5]; str+=s;
	s = " C/S2"; s += (LPCTSTR)m_IFMatrixStr[6]; str+=s;
	s = "   GB"; s += (LPCTSTR)m_IFMatrixStr[7]; str+=s;
	s = "   MB"; s += (LPCTSTR)m_IFMatrixStr[8]; str+=s;
	s = "   EB"; s += (LPCTSTR)m_IFMatrixStr[9]; str+=s;
	s = "Rud-1"; s += (LPCTSTR)m_IFMatrixStr[10]; str+=s;
	s = "Rud-2"; s += (LPCTSTR)m_IFMatrixStr[11]; str+=s;
	s = "Lei-1"; s += (LPCTSTR)m_IFMatrixStr[12]; str+=s;
	s = "Lei-2"; s += (LPCTSTR)m_IFMatrixStr[13]; str+=s;
	s = "         V    U   S1   K/S2  X  Lei-1 Lei-2\n"; str +=s;
	s = "                  Outputs";str += s;
}

// This routine unscrambles the IF Matrix data and sets or clears a bit in the
//  m_IFMatrixBits array that matches the display output format.
//  Each index into the bit array is a ROW.
//  Each bit in the raw corresponds to a column.
// Then the m_IFMatrixStr array is created using the bit structure in the bit array.
void CTLMdecode::CalcIFMatrix()
{
INT i,j,k;
DWORD bit;
DWORD ActCol = 0;
	for(i=0; i<14;i++)		//Initialize the arrays
	{
		for(j=0; j<35;j++)
			m_IFMatrixStr[i][j] = ' ';
		m_IFMatrixBits[i] = 0;
		m_IFMatrixStr[i][35] = 0x0A;
		m_IFMatrixStr[i][36] = 0x0;
	}
	//unscramble the telemetry bytes into a better format bit array
	SetBit(0x1F2,BIT0,1,BIT1);SetBit(0x1F2,BIT1,1,BIT2);SetBit(0x1F2,BIT2,1,BIT4);
	SetBit(0x1F2,BIT3,1,BIT3);SetBit(0x1F2,BIT4,1,BIT5);SetBit(0x1F2,BIT5,1,BIT6);
	SetBit(0x1F2,BIT6,10,BIT1);SetBit(0x1F2,BIT7,10,BIT2);
	SetBit(0x1F3,BIT0,10,BIT4);SetBit(0x1F3,BIT1,10,BIT3);SetBit(0x1F3,BIT2,10,BIT5);
	SetBit(0x1F3,BIT3,10,BIT6);
//
	SetBit(0x1F4,BIT0,0,BIT0);SetBit(0x1F4,BIT1,0,BIT2);SetBit(0x1F4,BIT2,0,BIT4);
	SetBit(0x1F4,BIT3,0,BIT3);SetBit(0x1F4,BIT4,0,BIT5);SetBit(0x1F4,BIT5,0,BIT6);
	SetBit(0x1F4,BIT6,2,BIT0);SetBit(0x1F4,BIT7,2,BIT2);
	SetBit(0x1F5,BIT0,2,BIT4);SetBit(0x1F5,BIT1,2,BIT3);SetBit(0x1F5,BIT2,2,BIT5);
	SetBit(0x1F5,BIT3,2,BIT6);
//
	SetBit(0x1F6,BIT0,12,BIT0);SetBit(0x1F6,BIT1,12,BIT1);SetBit(0x1F6,BIT2,12,BIT4);
	SetBit(0x1F6,BIT3,12,BIT3);SetBit(0x1F6,BIT4,12,BIT5);SetBit(0x1F6,BIT5,12,BIT6);
	SetBit(0x1F6,BIT6,5,BIT0);SetBit(0x1F6,BIT7,5,BIT1);
	SetBit(0x1F7,BIT0,5,BIT4);SetBit(0x1F7,BIT1,5,BIT3);SetBit(0x1F7,BIT2,5,BIT5);
	SetBit(0x1F7,BIT3,5,BIT6);
//
	SetBit(0x1F8,BIT0,11,BIT0);SetBit(0x1F8,BIT1,11,BIT1);SetBit(0x1F8,BIT2,11,BIT2);
	SetBit(0x1F8,BIT3,11,BIT3);SetBit(0x1F8,BIT4,11,BIT5);SetBit(0x1F8,BIT5,11,BIT6);
	SetBit(0x1F8,BIT6,13,BIT0);SetBit(0x1F8,BIT7,13,BIT1);
	SetBit(0x1F9,BIT0,13,BIT2);SetBit(0x1F9,BIT1,13,BIT3);SetBit(0x1F9,BIT2,13,BIT5);
	SetBit(0x1F9,BIT3,13,BIT6);
//
	SetBit(0x1FA,BIT0,9,BIT0);SetBit(0x1FA,BIT1,9,BIT1);SetBit(0x1FA,BIT2,9,BIT2);
	SetBit(0x1FA,BIT3,9,BIT4);SetBit(0x1FA,BIT4,9,BIT5);SetBit(0x1FA,BIT5,9,BIT6);
	SetBit(0x1FA,BIT6,7,BIT0);SetBit(0x1FA,BIT7,7,BIT1);
	SetBit(0x1FB,BIT0,7,BIT2);SetBit(0x1FB,BIT1,7,BIT4);SetBit(0x1FB,BIT2,7,BIT5);
	SetBit(0x1FB,BIT3,7,BIT6);
//
	SetBit(0x1FC,BIT0,4,BIT0);SetBit(0x1FC,BIT1,4,BIT1);SetBit(0x1FC,BIT2,4,BIT2);
	SetBit(0x1FC,BIT3,4,BIT4);SetBit(0x1FC,BIT4,4,BIT3);SetBit(0x1FC,BIT5,4,BIT6);
	SetBit(0x1FC,BIT6,8,BIT0);SetBit(0x1FC,BIT7,8,BIT1);
	SetBit(0x1FD,BIT0,8,BIT2);SetBit(0x1FD,BIT1,8,BIT4);SetBit(0x1FD,BIT2,8,BIT3);
	SetBit(0x1FD,BIT3,8,BIT6);
//
	SetBit(0x1FE,BIT0,6,BIT0);SetBit(0x1FE,BIT1,6,BIT1);SetBit(0x1FE,BIT2,6,BIT2);
	SetBit(0x1FE,BIT3,6,BIT4);SetBit(0x1FE,BIT4,6,BIT3);SetBit(0x1FE,BIT5,6,BIT5);
	SetBit(0x1FE,BIT6,3,BIT0);SetBit(0x1FE,BIT7,3,BIT1);
	SetBit(0x1FF,BIT0,3,BIT2);SetBit(0x1FF,BIT1,3,BIT4);SetBit(0x1FF,BIT2,3,BIT3);
	SetBit(0x1FF,BIT3,3,BIT5);
// Now create the graphic text lines connecting the active rows and columns
	for(i=0;i<14;i++)	// i is row address
	{
		if(m_IFMatrixBits[i] != 0)	// if at least one column is connected to this row
		{
			k = 5*7-1;
			bit = BIT0;							//bit = current column pointer
			for( bit = BIT6; bit>0; bit = bit>>1 )
			{
				if( (bit & m_IFMatrixBits[i]) != 0 )	//if row is connected to a column
				{
					m_IFMatrixStr[i][k] = '+';
					for(j=0;j<k;j++)
						m_IFMatrixStr[i][j] = '-';
				}
				k -= 5;
			}
		}
		bit = BIT0;
		ActCol |= m_IFMatrixBits[i];
		for(j=0; j<7; j++)	//Draw active columns
		{
			k = (j+1)*5-1;
			if( ActCol & bit )
				if(m_IFMatrixStr[i][k] != '+')
					m_IFMatrixStr[i][k] = '|';
			bit = bit<<1;
		}
	}
}

// copies m_pDat[inptr] bit 'inbit'  to  m_IFMatrixBits[outptr] bit 'outbit'
void CTLMdecode::SetBit(INT inptr, BYTE inbit, INT outptr, INT outbit)
{
	m_IFMatrixBits[outptr] |= ( (m_pDat[inptr]&inbit) ? outbit : 0);
}

const char* CTLMdecode::GetMsg(INT adr, INT bit, t_msg MsgArray)
{
const char* ptr;
	if( (m_pDat[adr] & bit) == 0 )
		ptr = MsgArray[0];
	else
		ptr = MsgArray[1];
	return ptr;
}

// returns string relating to two bits m_pDat['adr1'] & bit1
//  and m_pDat['adr2'] & bit2. The msg is from string 'MsgArray'
const char* CTLMdecode::GetMsg2(INT adr1,INT bit1,
							INT adr2,INT bit2, t_msg2 MsgArray)
{
const char* ptr;
INT indx=0;
	if( (m_pDat[adr1] & bit1) != 0 )
		indx |= BIT0;
	if( (m_pDat[adr2] & bit2) != 0 )
		indx |= BIT1;
	ptr = MsgArray[indx];
	return ptr;
}



void CTLMdecode::CopyData(PBYTE pBuf)
{
	for(INT i=0; i<512;i++)
		m_pDat[i] = pBuf[i];
}

void CTLMdecode::GetAmsatDay( CTime& ta )
{
INT year = AtoB(0x24,1000) + AtoB(0x25,100) +AtoB(0x26,10) + AtoB(0x27,1);
INT month = AtoB(0x29,10) + AtoB(0x2A,1);
INT day = AtoB(0x2C,10) + AtoB(0x2D,1);
INT hr = AtoB(0x30,10) + AtoB(0x31,1);
INT min = AtoB(0x33,10) + AtoB(0x34,1);
INT sec = AtoB(0x36,10) + AtoB(0x37,1);
	if( (year < 2000) || (year > 2038) )
		year = 2000;
	if( (month > 12) || (month < 1) )
		month = 6;
	if( (day > 31) || (day < 1) )
		day = 15;
	if( (hr > 24) || (hr < 0) )
		hr = 12;
	if( (min > 60) || (min < 0) )
		min = 30;
	if( (sec > 60) || (sec < 0) )
		sec = 30;
CTime tx( year, month, day, hr, min, sec ); // 00:00:00 Jan 1, 1978 AMSAT day 0
	ta = tx;
}

INT CTLMdecode::AtoB(INT x, INT mul)
{
	return (INT)(m_pDat[x] - '0') * mul;
}

double CTLMdecode::CalcSEUSens(void)
{
double x = (double)m_pDat[0x18A]*4800.0/256.0;
	if(m_pDat[0x18A]&BIT1)
		x -= .5;
	if(m_pDat[0x18A]&BIT0)
		x += 1.25;
	return x;
}
double CTLMdecode::CalcSunSens(void)
{
INT x = (m_pDat[0x1C6] >> 4) & 0x0F;
	return (double)x * 4.8 / 16.0;
}
double CTLMdecode::CalcWRPM(INT adr)
{
INT x = m_pDat[adr] + 256*m_pDat[adr+1] + 2;
double d = 1.0/(double)x - 4.069e-5;
	return  (d * 2.4e6 * 960.0/19.0);
}

const char* CTLMdecode::GetPointing(void)
{
const char* ptr;
INT x = m_pDat[0x1C6] & (BIT0|BIT1 );
	ptr = MSG_POINTING[x];
	return ptr;
}

double CTLMdecode::CalcBatOffset(void)
{
INT x = m_pDat[0x18C];
	if( x<64 )
		x += 256;
	return( (double)x*.04 + 17.76 );
}

double CTLMdecode::CalcBCROffset(INT adr)
{
INT x = m_pDat[adr];
	if( x<128 )
		x += 256;
	return( (double)x*.1 - 5.6 );

}

double CTLMdecode::CalcSunAngle( void )
{
BYTE val = m_pDat[0x191];
BOOL NegAngle = FALSE;
	if(val > 127)
	{
		val = 256-val;
		NegAngle = TRUE;
	}
double sa = (180.0/PI)*atan( tan(60.0*PI/180.0) * 
			sin(PI*(double)val/256.0) ); 
	if( NegAngle )
		sa = -sa;
	return sa;
}
